//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import "CustomDatePicker.h"
#import "CustomDatePickerViewController.h"


@interface CustomDatePicker ()

{
    UIToolbar *actionItemsToolBar;
    UIDatePicker *datePicker;
    UILabel *customDatePickerTitle;
    CustomDatePickerViewController *customDatePickerController;
    
}

@property (nonatomic ,copy) CustomDateDoneBlock customDatePickerDone;
@property (nonatomic ,copy) CustomDateCancelBlock customDatePickerCancel;

@end

@implementation CustomDatePicker


- (instancetype)initWithTitle : (NSString *)title
                    titleFont : (UIFont *)titleFont
               datePickerMode : (UIDatePickerMode)datePickerMode
                 selectedDate : (NSDate *)selectedDate
                    doneBlock : (CustomDateDoneBlock)doneBlock
                  cancelBlock : (CustomDateCancelBlock)cancelBlock {
    
    self = [super initWithFrame:CGRectZero];
    
    if (self) {
        
        actionItemsToolBar = [[UIToolbar alloc] init];
        actionItemsToolBar.barStyle = UIBarStyleDefault;
        [actionItemsToolBar sizeToFit];
        
        NSMutableArray *toolBarItems = [NSMutableArray new];
        
        UIBarButtonItem *clearButton = [[UIBarButtonItem alloc] initWithTitle:NSLocalizedString(@"Clear", nil) style:UIBarButtonItemStyleDone target:self action:@selector(customPickerCancelClicked:)];
        
        [toolBarItems addObject:clearButton];
        
        //  Create a fake button to maintain flexibleSpace between clear button and custom picker title
        UIBarButtonItem *leftNilButton =[[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace target:nil action:nil];
        [toolBarItems addObject:leftNilButton];
        
        // Create a label for title
        customDatePickerTitle = [[UILabel alloc] initWithFrame:CGRectMake(0, 0, actionItemsToolBar.frame.size.width/2, actionItemsToolBar.frame.size.height)];
        customDatePickerTitle.font = titleFont;
        customDatePickerTitle.textAlignment = NSTextAlignmentCenter;
        customDatePickerTitle.text = title;
        customDatePickerTitle.textColor = [UIColor blackColor];
        customDatePickerTitle.autoresizingMask = UIViewAutoresizingFlexibleWidth;
        
        UIBarButtonItem *custumTitleButton = [[UIBarButtonItem alloc] initWithCustomView:customDatePickerTitle];
        [toolBarItems addObject:custumTitleButton];
        
        //  Create a fake button to maintain flexibleSpace between title and done button.
        UIBarButtonItem *rightNilButton =[[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace target:nil action:nil];
        [toolBarItems addObject:rightNilButton];
        
        UIBarButtonItem *doneButton =[[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemDone target:self action:@selector(customPickerDoneClicked:)];
        [toolBarItems addObject:doneButton];
        
        [actionItemsToolBar setItems:toolBarItems];

        [self addSubview:actionItemsToolBar];
        
    //UIDatePicker
        datePicker = [[UIDatePicker alloc] initWithFrame:CGRectMake(0, CGRectGetMaxY(actionItemsToolBar.frame), CGRectGetWidth(actionItemsToolBar.frame), [[UIScreen mainScreen] bounds].size.height/3)];
        [datePicker addTarget:self action:@selector(dateChanged:) forControlEvents:UIControlEventValueChanged];
        [datePicker setDatePickerMode:datePickerMode];
        [datePicker setDate:selectedDate];
        [self addSubview:datePicker];
        
    //Initial settings
        self.backgroundColor = [UIColor colorWithRed:247.f/255.f green:247.f/255.f blue:247.f/255.f alpha:1.0f];
        [self setFrame:CGRectMake(0, 0, CGRectGetWidth(datePicker.frame), CGRectGetMaxY(datePicker.frame))];
        
        self.autoresizingMask = UIViewAutoresizingFlexibleTopMargin|UIViewAutoresizingFlexibleWidth;
        actionItemsToolBar.autoresizingMask = UIViewAutoresizingFlexibleWidth;
        datePicker.autoresizingMask = UIViewAutoresizingFlexibleWidth;
        
        self.customDatePickerDone = doneBlock;
        self.customDatePickerCancel = cancelBlock;
    }
    
    return self;

}

- (void) showPickerView {
    
    customDatePickerController = [[CustomDatePickerViewController alloc] init];
    [customDatePickerController showPickerView:self completion:nil];
    
}

#pragma mark - Done/Cancel action event

-(void)customPickerCancelClicked:(UIBarButtonItem*)barButton {
     __weak CustomDatePicker *blocksafeSelf = self;
    
    [customDatePickerController dismissWithCompletion:^{
        blocksafeSelf.customDatePickerCancel(blocksafeSelf);
    }];
    
}

-(void)customPickerDoneClicked:(UIBarButtonItem*)barButton {
    __weak CustomDatePicker *blocksafeSelf = self;
    
    [customDatePickerController dismissWithCompletion:^{
        blocksafeSelf.customDatePickerDone(blocksafeSelf ,[datePicker date]);
    }];
}

-(void)dateChanged:(CustomDatePicker*)customDatePicker {

}


@end
