//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import "UIViewController+Share.h"
#import "UIViewController+Alert.h"

@implementation UIViewController (Share)

- (void)didBeginDoingShareAction:(ShareAction)action {
    // abstract method
}

- (void)didFinishDoingAction:(ShareAction)action onShare:(id)share result:(id)result response:(id)response {
    // abstract method
}

#pragma mark -
#pragma mark Cancel
- (void)addShareCancelAction:(UIAlertController *)alertController {
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Cancel", nil)
                    style:UIAlertActionStyleCancel
                andAction:nil];
}

- (void)executeAction:(ShareAction)action forShare:(PCShareItem *)share {
    switch (action) {
        case deleteShareAction:
            [self deleteShareActionCompletion:share];
            break;
    
        default:
            break;
    }

}

- (void)showActionsForSelectedShare:(PCShareItem *)selectedShare {
    
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:[NSString stringWithFormat:@"%@ (%@)",selectedShare.name,selectedShare.totalResourceCount]
                                                                             message:nil
                                                                      preferredStyle:UIAlertControllerStyleActionSheet];
    
    
    [self addDeleteShareAction:alertController forSelectedResource:selectedShare];
    [self addShareCancelAction:alertController];
    
    [self presentViewController:alertController animated:YES completion:nil];
    
}

- (void)deleteShare:(PCShareItem *)shareItem {
    
    __weak UIViewController *blocksafeSelf = self;
    /*
     @Description -
     This API is used to delete the user created share. Once the deletion is done/failed result will come with completion block.
     
     @Parameters -
     PCDeleteShareAPIParams : This class store all the parameters required to delete a share.
     * shareID and accessToken are required parameter to call this API. For providing access token to SDK user have to logged In. If user will not pass required parameter, API will throw an NSInvalidArgumentException and user have to handle it.
     * There is no optional parameter for this API.
     * More information about parameters you will find in PCDeleteShareAPIParams.h class.
     
     @CompletionHandler -
     This completion block gets called once share deletion process is done/failed. completion block contains below parameter.
     PCAPIQueryResponse : Stores all the server response and error for this API.
     
     @To delete users share, you must take the following steps: -
     * Include VerizonCloudAPI.framework header file in your class.
        #import <VerizonCloudAPI/VerizonCloudAPI.h>
     * Initalize the PCDeleteShareAPIParams class and provide the required parameter.
     * Initalize the PCAPISDK object and call the deleteShare:withCompletionHandler: API.
     * Get the result(Success or failed) in completion block.
     * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
     */
    
    // Sample code for deleteShare:withCompletionHandler: API
    
    // Initalize PCDeleteShareAPIParams class.
    PCDeleteShareAPIParams *deleteParams = [[PCDeleteShareAPIParams alloc] init];
    deleteParams.shareUID = shareItem.UID; // provide  value to the required parameter.
    
    // Initalize the PCAPISDK class and call the deleteShare:withCompletionHandler: API.
    [[PCAPISDK sharedInstance] deleteShare:deleteParams withCompletionHandler:^(PCAPIQueryResponse *response) {
        [blocksafeSelf didFinishDoingAction:deleteShareAction onShare:shareItem result:nil response:response];
    }];
}

- (void)deleteShareActionCompletion:(PCShareItem *)shareItem {
    __weak UIViewController *blocksafeSelf = self;
    
    NSString *message = NSLocalizedString(@"Do you want to delete this Share", nil);
    
    [self showOKCancelAlertWithTitle:NSLocalizedString(@"Warning", nil)
                          andMessage:message
                            okAction:^(UIAlertAction *action) {
                                [blocksafeSelf didBeginDoingShareAction:deleteShareAction];
                                [blocksafeSelf deleteShare:shareItem];
                            }
                        cancelAction:^(UIAlertAction *action) {
                            
                        }];
}

- (void)addDeleteShareAction:(UIAlertController *)alertController forSelectedResource:(PCShareItem *)selectedShare {
    __weak UIViewController *blocksafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Delete", nil)
                    style:UIAlertActionStyleDestructive
                andAction:^(UIAlertAction * _Nonnull action) {
                    
                    [blocksafeSelf deleteShareActionCompletion:selectedShare];
                }];
}

@end
