//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import "PCContact+Additions.h"
#import "ImageDownloadView.h"
#import <UIKit/UIKit.h>

@implementation PCContact (Additions)

- (NSString *)getAnyPhoneNumber {

    if (self.telphones.count) {
        
        PCTelContactType *telephone = [self.telphones objectAtIndex:0];
        return [NSString stringWithFormat:@"%@", telephone.number];
    }
    return nil;
}

- (NSString *)getAnyEmail {
    
    if (self.emails.count) {
        
        PCEmailContactType *email = [self.emails objectAtIndex:0];
        return [NSString stringWithFormat:@"%@", email.email];
    }
    return nil;
}

- (NSString *)getDisplaytext {

    if (self.firstName.length) {
       
        return self.firstName;
        
    } else if (self.lastName.length) {
       
        return self.lastName;
        
    } else if (self.orgName.length) {
        
        return self.orgName;
        
    } else if ([self getAnyEmail].length) {
       
        return [self getAnyEmail];
        
    } else if ([self getAnyPhoneNumber].length) {
       
        return [self getAnyPhoneNumber];
        
    }
    
    return NSLocalizedString(@"Unknown", nil);
}

- (CNMutableContact *)getCNContact {

    CNMutableContact *CNContact = [CNMutableContact new];
    CNContact.givenName = self.firstName;
    CNContact.familyName = self.lastName;
    CNContact.organizationName = self.orgName;
    CNContact.jobTitle = self.jobTitle;
    CNContact.namePrefix = self.namePrefix;
    CNContact.nickname = self.nickname;
    CNContact.note = self.note;
    
    CNContact.imageData = UIImagePNGRepresentation([UIImage imageNamed:@"Contact"]);
    
    if (self.telphones.count) {
        
        NSMutableArray *CNTelephoneArray = [NSMutableArray new];
        for (PCTelContactType *telephone in self.telphones) {
            
            CNLabeledValue *CNtelephone = [CNLabeledValue labeledValueWithLabel:telephone.type value:[CNPhoneNumber phoneNumberWithStringValue:[NSString stringWithFormat:@"%@", telephone.number]]];

            [CNTelephoneArray addObject:CNtelephone];
        }
        
        CNContact.phoneNumbers = CNTelephoneArray;
    }
    
    if (self.emails.count) {
        
        NSMutableArray *CNEmailArray = [NSMutableArray new];
        
        for (PCEmailContactType *email in self.emails) {
            
            CNLabeledValue *CNEmail = [CNLabeledValue labeledValueWithLabel:email.type value:email.email];
            [CNEmailArray addObject:CNEmail];
        }
        
        CNContact.emailAddresses = CNEmailArray;
    }
    
    if (self.imArray.count) {
        
        NSMutableArray *CNIMArray = [NSMutableArray new];
        
        for (PCInstantMessageType *im in self.imArray) {
            CNLabeledValue *CNIm = [CNLabeledValue labeledValueWithLabel:im.type value:[[CNInstantMessageAddress alloc] initWithUsername:im.address service:im.type]];
            [CNIMArray addObject:CNIm];
        }
        
        CNContact.instantMessageAddresses = CNIMArray;
    }
    
    if (self.addresses.count) {
        
        NSMutableArray *CNAddressArray = [NSMutableArray new];
        
        for (PCAddressType *address in self.addresses) {
            CNMutablePostalAddress *postalAddress = [CNMutablePostalAddress new];
            postalAddress.street = address.street;
            postalAddress.city = address.city;
            postalAddress.postalCode = address.pobox;
            postalAddress.country = address.country;

            CNLabeledValue *CNAddress = [CNLabeledValue labeledValueWithLabel:address.type value:postalAddress];
            [CNAddressArray addObject:CNAddress];
        }
        
        CNContact.postalAddresses = CNAddressArray;
    }
    
    return CNContact;
}

- (NSString *)thumbnailCachePath {
    
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentsDirectory = [paths objectAtIndex:0];
    NSString *thumbnailImagePath = [documentsDirectory stringByAppendingPathComponent:[NSString stringWithFormat:@"Thumbs_%@.jpg", self.photoReference]];
    return thumbnailImagePath;
}

- (UIImage *)defaultPic {
    
    return [UIImage imageNamed:@"Contact"];
}

- (void)setAvatarForImageView:(ImageDownloadView *)imageView withCompletionHandler:(void (^)(BOOL didSetImage))completion {

    if (self.photoReference.length) {
        
        imageView.imageID = self.photoReference; // imageID needs to be set to encounter cell reusability feature
        
        UIImage *thumbnailImage = [UIImage imageWithContentsOfFile:[self thumbnailCachePath]];
        
        if (thumbnailImage) {
            
            imageView.image = thumbnailImage;
            completion(YES);

        } else {
            
            // Intialize the PCThumbnailAPIParams object.
            PCThumbnailAPIParams *params = [[PCThumbnailAPIParams alloc] init];
            params.th = 320;
            params.tw = 320;
            params.contentToken = self.photoReference;
            
            imageView.image = [self defaultPic];
            
            __weak PCContact *blocksafeSelf = self;
            /*
             @Description :
             This API is used to download thumbnail image of resource. Once the download process is succeed/failed result will come with completion block.
             
             @Parameters :
             PCThumbnailAPIParams:
             * This class stores all the parameters required to download thumbnail image of resource. It contains attributes such as contentToken, size, th and tw.
             * For providing access token to SDK user have to logged In.
             * contentToken and Either 'size' or ('th' and 'tw') is required parameter to call this API.
             * value for size are (Thumbnail size: 'xs' (24x24), 's' (64x64), or 'm' (128x128).
             * If user will not provide required parameter, API will throw an NSInvalidArgumentException and user have to handle it.
             * More information about parameters you will find in PCThumbnailAPIParams class.
             
             @CompletionHandler :
             This completion block gets called once the download process is succeed/failed. completion block contains below parameter.
             PCAPIQueryResponse : Stores all the server response and error for this API. dowloaded result comes in downloadedURL attribute.
             
             @To download thumbnail image of resource, you must take the following steps:
             * Include VerizonCloudAPI.framework header file in your class.
             #import <VerizonCloudAPI/VerizonCloudAPI.h>
             * Initalize the PCThumbnailAPIParams class and provide the required parameter.
             * Initalize the PCAPISDK object and call the downloadThumbnail:withCompletionHandler: API.
             * Perform the downloadThumbnail:withCompletionHandler: API call using the code sample below.
             * Get the result (succeed or failed) in completion block.
             * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
             */
            
            // Initilize the PCAPISDK and call the API.
            
            PCAPI *thumbnailAPI = [[PCAPISDK sharedInstance] downloadThumbnail:params withCompletionHandler:^(PCAPIQueryResponse *response) {
                // Get the result in this block.
                NSData *fileData = [NSData dataWithContentsOfURL:response.downloadedURL];
                if (fileData) {
                    [fileData writeToFile:[blocksafeSelf thumbnailCachePath] atomically:YES];
                }
                
                
                if ([imageView.imageID isEqualToString:blocksafeSelf.photoReference]) { // cell is still representing same resource
                    
                    
                    if (fileData) {
                        
                        UIImage *thumbImage = [UIImage imageWithContentsOfFile:[blocksafeSelf thumbnailCachePath]];
                        if (thumbImage) {
                            
                            imageView.image = thumbImage;
                            completion(YES);
                        } else {
                            
                            imageView.image = [blocksafeSelf defaultPic];
                            completion(YES);
                        }
                        
                    } else {
                        
                        imageView.image = [blocksafeSelf defaultPic];
                        completion(YES);
                    }
                } else {
                
                    completion(NO);
                }
            }];
            
            imageView.api = thumbnailAPI;
        }
    } else {
        
        imageView.image = [self defaultPic];
        completion(YES);
    }
}

@end
