//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import "PlaylistItemViewController.h"
#import "UIViewController+Alert.h"
#import "UIViewController+ResourceActions.h"
#import "PCResource+Info.h"
#import "ImageGridCell.h"
#import "PCPlaylist+Additions.h"

@interface FileGridViewController (Protected)

@property (nonatomic, strong) UIRefreshControl *refreshControl;
@property (nonatomic, strong) UICollectionView *gridView;
@property (nonatomic, strong) NSString *emptyViewMessage;

- (void)refreshView;

@end

@interface PlaylistItemViewController ()


@end

@implementation PlaylistItemViewController


- (void)viewDidLoad {

    [super viewDidLoad];
    
    self.title = self.playlist.name;
    
    switch (self.playlist.playlistType) {
        case playlistTypeMusic:
            self.emptyViewMessage = NSLocalizedString(@"No items in this playlist", nil);
            break;
        case playlistTypeImage:
        case playlistTypeVideo:
        case playlistTypeImageVideo:
            self.emptyViewMessage = NSLocalizedString(@"No items in this Album", nil);
            break;
        default:
            self.emptyViewMessage = NSLocalizedString(@"No items", nil);
            break;
    }
    
    
    [self.refreshControl beginRefreshing];
    self.gridView.alwaysBounceVertical = YES;
    [self reloadData];
}

- (void)reloadData {
    PCGetPlaylistItemsAPIParams *params = [[PCGetPlaylistItemsAPIParams alloc] init];
    params.playlistUID = self.playlist.UID;
    
    __weak PlaylistItemViewController *blocksafeSelf = self;
    
    [_playlist getFileWithCompletionHandler:^(NSArray *playlistFiles, PCAPIQueryResponse *response) {
        [blocksafeSelf.refreshControl endRefreshing];
        
        switch (response.urlResponse.statusCode) {
            case PCAPIResponseCodeOK: {
                
                self.dataSource = [NSArray arrayWithArray:playlistFiles];
                [blocksafeSelf refreshView];
            }
                break;
            case PCAPIResponseCodeUnauthorized:
                [blocksafeSelf showUnAuthorizedAlert];
                break;
            case PCAPIResponseCodeServiceUnavailable:
                [blocksafeSelf showServerUnavailableAlert];
                break;
            default:
                [blocksafeSelf showOperationFailedAlert:NSLocalizedString(@"Playlist item fetch", nil)];
                break;
        }

    }];
}

#pragma mark -

- (void)gridCellDidLongPressed:(ImageGridCell *)cell {
    NSIndexPath *indexPath = [self.gridView indexPathForCell:cell];
    PCResource *selectedResource = [self.dataSource objectAtIndex:indexPath.row];
#if HUBBLE_MEDIA_BROWSER
    
    if (selectedResource.fileType == musicFileType) {
        [self resourceSelected:selectedResource showActions:[NSNumber numberWithInt:fileInfoResourceAction], [NSNumber numberWithInt:addToPlaylistResourceAction], [NSNumber numberWithInt:removeFromPlaylistResourceAction], nil];
        
    } else if (selectedResource.fileType == imageFileType || selectedResource.fileType == videoFileType) {
        [self resourceSelected:selectedResource showActions:[NSNumber numberWithInt:fileInfoResourceAction], [NSNumber numberWithInt:addToAlbumResourceAction], [NSNumber numberWithInt:removeFromAlbumResourceAction], nil];
    }
    
#endif
}


#pragma mark -

- (id)dataSourceForResourceAction:(ResourceAction)action {
    switch (action) {
        case removeFromPlaylistResourceAction:
        case removeFromAlbumResourceAction:
            return self.playlist.UID;
            break;
            
        default:
            return nil;
            break;
    }
}


@end
