//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import "ResourceGroupByViewController.h"
#import "UIViewController+Dismiss.h"
#import "UIViewController+Alert.h"
#import "UIViewController+RightBarButtonAction.h"
#import "UIViewController+PlaylistActions.h"
#import "UIViewController+Spinner.h"
#import "CustomTableViewCell.h"
#import "PCAccountInfo+additions.h"
#import "NSArray+Addition.h"


@interface BaseTableViewController (Private)

@property (nonatomic, strong) NSString *emptyMessage;

@end

@interface ResourceGroupByViewController () <UITableViewDataSource>

@property (nonatomic, strong) NSArray *resourceList;

@end

@implementation ResourceGroupByViewController

- (void)viewDidLoad {

    [super viewDidLoad];
    
     [self addCancelButton];
    
    self.refreshControl = [[UIRefreshControl alloc] init];
    [self.refreshControl addTarget:self action:@selector(reloadData) forControlEvents:UIControlEventValueChanged];
    [self.tableView addSubview:self.refreshControl];
    
    [self.refreshControl beginRefreshing];
    
    [self addRightBarButtonItemWithActions:[NSNumber numberWithInt:accountInfoRightBarAction], [NSNumber numberWithInt:searchRightBarAction], [NSNumber numberWithInt:signOutRightBarAction], nil];
    
    [self reloadData];

}

- (void)reloadData {
    
    __weak ResourceGroupByViewController *blocksafeSelf = self;
    
    switch (self.resourceGroupType) {
        case resourceGroupTypeAlbum: {

            /*
             @Description :
             This API is used to fetch all audio album from the account. Once the fetch process is succeed/failed result will come with completion block.
             
             @Parameters: nil
             * For providing access token to SDK user have to logged In.
             
             @CompletionHandler :
             This completion block gets called once fetch process is succeed/failed. completion block contains below parameter.
             PCAPIQueryResponse :
             * Stores all the server response and error for this API.
             NSArray :
             * An array of PCAlbum object. PCAlbum class stores the album information of a audio file.
             * Using the PCAlbum object we can get all file using getFileWithCompletionHandler method. This method fetch all PCAudio file related to this PCAlbum object.
             
             @To fetch all audio album from the account, you must take the following steps:
             * Include VerizonCloudAPI.framework header file in your class.
             #import <VerizonCloudAPI/VerizonCloudAPI.h>
             * Initalize the PCAPISDK object and call the getAudioAlbumsWithCompletionHandler: API.
             * Perform the getAudioAlbumsWithCompletionHandler: API call using the code sample below.
             * Get the result (succeed or failed) in completion block.
             * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
             */
            
            // Sample code for getAudioAlbumsWithCompletionHandler: API
            [[PCAPISDK sharedInstance] getAudioAlbumsWithCompletionHandler:^(NSArray *audioAlbumList, PCAPIQueryResponse *response) {
                
                [self.refreshControl endRefreshing];
                
                switch (response.urlResponse.statusCode) {
                        
                    case PCAPIResponseCodePreconditionFailed:
                    case PCAPIResponseCodeOK: {
                        blocksafeSelf.resourceList = audioAlbumList;
                        [blocksafeSelf refreshView];
                    }
                        break;
                    case PCAPIResponseCodeUnauthorized:
                        [blocksafeSelf showUnAuthorizedAlert];
                        break;
                    case PCAPIResponseCodeServiceUnavailable:
                        [blocksafeSelf showServerUnavailableAlert];
                        break;
                    default:
                        [blocksafeSelf showOperationFailedAlert:NSLocalizedString(@"Audio album fetch", nil)];
                        break;
                }
            }];
        }
            break;
        case resourceGroupTypeArtist: {
            
            /*
             @Description :
             This API is used to fetch all artist from the account. Once the fetch process is succeed/failed result will come with completion block.
             
             @Parameters: nil
             * For providing access token to SDK user have to logged In.
             
             @CompletionHandler :
             This completion block gets called once fetch process is succeed/failed. completion block contains below parameter.
             PCAPIQueryResponse :
             * Stores all the server response and error for this API.
             NSArray :
             * An array of PCArtist object. PCArtist class stores the Artist information of a audio file.
             * Using the PCArtist object we can get all file using getFileWithCompletionHandler method. This method fetch all PCAudio file related to this PCArtist object.
             
             @To fetch all artist from the account, you must take the following steps:
             * Include VerizonCloudAPI.framework header file in your class.
             #import <VerizonCloudAPI/VerizonCloudAPI.h>
             * Initalize the PCAPISDK object and call the getArtistsWithCompletionHandler: API.
             * Perform the getArtistsWithCompletionHandler: API call using the code sample below.
             * Get the result (succeed or failed) in completion block.
             * If session is no longer valid , call the logout API which remove access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
             */
            
            // Sample code for getArtistsWithCompletionHandler: API

            [[PCAPISDK sharedInstance] getArtistsWithCompletionHandler:^(NSArray *artistList, PCAPIQueryResponse *response) {
                
                [self.refreshControl endRefreshing];
                
                switch (response.urlResponse.statusCode) {
                        
                    case PCAPIResponseCodePreconditionFailed:
                    case PCAPIResponseCodeOK: {
                            blocksafeSelf.resourceList = artistList;
                            [blocksafeSelf refreshView];
                    }
                        break;
                    case PCAPIResponseCodeUnauthorized:
                        [blocksafeSelf showUnAuthorizedAlert];
                        break;
                    case PCAPIResponseCodeServiceUnavailable:
                        [blocksafeSelf showServerUnavailableAlert];
                        break;
                    default:
                        [blocksafeSelf showOperationFailedAlert:NSLocalizedString(@"Artist fetch", nil)];
                        break;
                }
            }];
        }
            break;
        case resourceGroupTypeGenre: {
            
            /*
             @Description :
             This API is used to fetch all genre from the account. Once the fetch process is succeed/failed result will come with completion block.
             
             @Parameters: nil
             * For providing access token to SDK user have to logged In.
             
             @CompletionHandler :
             This completion block gets called once fetch process is succeed/failed. completion block contains below parameter.
             PCAPIQueryResponse :
             * Stores all the server response and error for this API.
             NSArray :
             * An array of PCGenre object. PCGenre class stores the Genre information of a audio file.
             * Using the PCGenre object we can get all file using getFileWithCompletionHandler method. This method fetch all PCAudio file related to this PCGenre object.
             
             @To fetch all genre from the account, you must take the following steps:
             * Include VerizonCloudAPI.framework header file in your class.
             #import <VerizonCloudAPI/VerizonCloudAPI.h>
             * Initalize the PCAPISDK object and call the getGenresWithCompletionHandler: API.
             * Perform the getGenresWithCompletionHandler: API call using the code sample below.
             * Get the result (succeed or failed) in completion block.
             * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
             */
            
            // Sample code for getGenresWithCompletionHandler: API

            [[PCAPISDK sharedInstance] getGenresWithCompletionHandler:^(NSArray *genreList, PCAPIQueryResponse *response) {
                
                [self.refreshControl endRefreshing];
                
                switch (response.urlResponse.statusCode) {
                        
                    case PCAPIResponseCodePreconditionFailed:
                    case PCAPIResponseCodeOK: {
                        blocksafeSelf.resourceList = genreList;
                        [blocksafeSelf refreshView];
                    }
                        break;
                    case PCAPIResponseCodeUnauthorized:
                        [blocksafeSelf showUnAuthorizedAlert];
                        break;
                    case PCAPIResponseCodeServiceUnavailable:
                        [blocksafeSelf showServerUnavailableAlert];
                        break;
                    default:
                        [blocksafeSelf showOperationFailedAlert:NSLocalizedString(@"Genre fetch", nil)];
                        break;
                }
            }];
        }
            break;
        case resourceGroupTypeUnknown:
            break;
    }
}
- (void)refreshView {
    
    if (self.resourceList.count) {
        [self.tableView reloadData];
    } else {
        [self showOKAlertWithTitle:NSLocalizedString(@"Empty", nil)
                        andMessage:self.emptyMessage attributedString:nil okAction:nil];
    }
}

#pragma mark -
#pragma mark Right bar actions

- (void)didBeginDoingRightBarButtonAction:(rightBarAction)action {
    [self showSpinner];
}

- (void)didFinishDoingRightBarButtonAction:(rightBarAction)action :(id)result :(id)response {
    
    [self hideSpinner];
    
    PCAPIQueryResponse *APIResponse = (PCAPIQueryResponse *)response;
    
    switch (APIResponse.urlResponse.statusCode) {
            
        case PCAPIResponseCodeResourceCreated:
        case PCAPIResponseCodeOK: {
            
            switch (action) {
                case accountInfoRightBarAction: {
                    
                    PCAccountInfo *accountInfo = (PCAccountInfo *)result;
                    [self showOKAlertWithTitle:[self stringForRightBarAction:action]
                                    andMessage:[accountInfo humanReadableAccountInfo]
                              attributedString:nil
                                      okAction:nil];
                }
                    break;
                    break;
                default:
                    break;
            }
        }
            break;
        case PCAPIResponseCodeUnauthorized:
            [self showUnAuthorizedAlert];
            break;
        case PCAPIResponseCodeServiceUnavailable:
            [self showServerUnavailableAlert];
            break;
        default:
            [self showOperationFailedAlert:[self stringForRightBarAction:action]];
            break;
    }
}



@end
