//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import <XCTest/XCTest.h>
#import "PCTestConstants.h"

@interface PCCreatePlaylistAPITests : XCTestCase

@end

@implementation PCCreatePlaylistAPITests

- (void)setUp {
    [super setUp];
    // Put setup code here. This method is called before the invocation of each test method in the class.
}

- (void)tearDown {
    // Put teardown code here. This method is called after the invocation of each test method in the class.
    [super tearDown];
}

- (NSString*)valueBasedOnPlaylistType:(playlistType)playlistType {
    
    NSString *playlistTypeValue;
    
    switch (playlistType) {
        case playlistTypeMusic:
            playlistTypeValue = @"music";
            break;
        case playlistTypeImage:
            playlistTypeValue = @"image";
            break;
        case playlistTypeVideo:
            playlistTypeValue = @"video";
            break;
        case playlistTypeImageVideo:
            playlistTypeValue = @"image-video";
            break;
        case playlistTypeUnknown:
            break;
    }
    return playlistTypeValue;
}

/*!
 * Test Case: auth token is a required parameter. SDK should throw an exception in case auth token is nil.
 */
- (void)testNilAuthToken {
    
    // If User is not logged in, createPlaylist api should throw an exception
    if (![[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        PCCreateplaylistAPIParams *params = [[PCCreateplaylistAPIParams alloc] init];
        
        XCTAssertThrows([[PCAPISDK sharedInstance] createPlaylist:params withCompletionHandler:^(PCPlaylist *playlist, PCAPIQueryResponse *response) {}],@"Get create playlist API with nil access token");
    }
}

/*!
 * Test PlaylistType is required to create playlist response,otherwise test will fail.
 */

- (void)testRequiredPlaylistTypeToCreatePlaylist {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        PCCreateplaylistAPIParams *params = [[PCCreateplaylistAPIParams alloc] init];
        params.name = @"test playlist";
        
        XCTAssertThrows([[PCAPISDK sharedInstance] createPlaylist:params withCompletionHandler:^(PCPlaylist *playlist, PCAPIQueryResponse *response) {}],@"Create playlist API throw exception without PlaylistType");
        
        
    } else {
        
        XCTFail(@"Test Failed due to nil access token");
        
    }
}

/*!
 * Test PlaylistName is required to create playlist response,otherwise test will fail.
 */

- (void)testRequiredPlaylistNameToCreatePlaylist {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        PCCreateplaylistAPIParams *params = [[PCCreateplaylistAPIParams alloc] init];
        params.type = [self valueBasedOnPlaylistType:playlistTypeMusic];
        
        XCTAssertThrows([[PCAPISDK sharedInstance] createPlaylist:params withCompletionHandler:^(PCPlaylist *playlist, PCAPIQueryResponse *response) {}],@"Create playlist API throw exception without PlaylistName");
        
        
    } else {
        
        XCTFail(@"Test Failed due to nil access token");
        
    }
}

/*!
 * Test Case: If response code is 200, playlist object has to be of class PCPlaylist type. In all other cases (error), playlist items has to be returned nil.
 */

- (void)testCreatePlaylistWithoutItems {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        PCCreateplaylistAPIParams *params = [[PCCreateplaylistAPIParams alloc] init];
        params.type = [self valueBasedOnPlaylistType:playlistTypeMusic];
        params.name = @"Test Playlist";
        
        XCTestExpectation *expectation = [self expectationWithDescription:@"Create playlist API Result"];
        
        [[PCAPISDK sharedInstance] createPlaylist:params withCompletionHandler:^(PCPlaylist *playlist, PCAPIQueryResponse *response) {
            
            if (response.urlResponse.statusCode != PCAPIResponseCodeResourceCreated) {
                XCTFail(@"Playlist is not created");
            }
            [expectation fulfill];
            
        }];
        [self waitForExpectationsWithTimeout:ExpectationTimeOut handler:^(NSError *error) {
            
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
        }];
        
    } else {
         XCTFail(@"Test Failed due to nil access token");
    }
    
}

/*!
* Test Case: If response code is 200, playlist object has to be of class PCPlaylist type. In all other cases (error), playlist items has to be returned nil.
*/

- (void) testCreatePlaylistWithItems {
    
     if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
         
        XCTestExpectation *expectation = [self expectationWithDescription:@"Create playlist API Result"];
         PCSearchAPIParams *musicSearchParams = [[PCSearchAPIParams alloc] init];
         musicSearchParams.query = [NSString setValue:@"audio*" forKey:@"contentType"];
         [musicSearchParams setSortKey:@"name" isAscending:YES];
         
         [[PCAPISDK sharedInstance] search:musicSearchParams withCompletionHandler:^(NSArray *results, PCAPIQueryResponse *response) {
             
             if (response.urlResponse.statusCode == PCAPIResponseCodeOK || response.urlResponse.statusCode == PCAPIResponseCodePreconditionFailed) {
                 
                 PCResource *resource = [results objectAtIndex:0];
                 
                 PCCreateplaylistAPIParams *params = [[PCCreateplaylistAPIParams alloc] init];
                 params.type = [self valueBasedOnPlaylistType:playlistTypeMusic];
                 params.name = @"Test Playlist";
                 params.paths = @[resource.fullPath];
                 
                 [[PCAPISDK sharedInstance] createPlaylist:params withCompletionHandler:^(PCPlaylist *playlist, PCAPIQueryResponse *response) {
                      switch (response.urlResponse.statusCode) {
                         case PCAPIResponseCodeResourceCreated:
                             XCTAssertTrue([playlist isMemberOfClass:[PCPlaylist class]],@"Create playlist API return PCPlaylist object");
                             break;
                             
                         default:
                             XCTFail(@"Playlist is not created");
                             break;
                     }
                     [expectation fulfill];
                 }];
                 
             } else {
                 XCTFail(@"Test Failed due to absence of file");
             }
             [expectation fulfill];
         }];
         [self waitForExpectationsWithTimeout:ExpectationTimeOut*2 handler:^(NSError *error) {
             
             if (error) {
                 NSLog(@"Timeout Error: %@", error);
             }
         }];
     } else {
       XCTFail(@"Test Failed due to nil access token");
     }
    
}

@end
