//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import <XCTest/XCTest.h>
#import "PCTestConstants.h"

@interface PCGetPlaylistContentAPITests : XCTestCase

@end

@implementation PCGetPlaylistContentAPITests

- (void)setUp {
    [super setUp];
    // Put setup code here. This method is called before the invocation of each test method in the class.
}

- (void)tearDown {
    // Put teardown code here. This method is called after the invocation of each test method in the class.
    [super tearDown];
}

/*!
 * Test Case: auth token is a required parameter. SDK should throw an exception in case auth token is nil.
 */
- (void)testNilAuthToken {
    
    if (![[PCAPISDK sharedInstance] isUserLoggedIn]) {
        PCGetPlaylistContentAPIParams *params = [[PCGetPlaylistContentAPIParams alloc] init];
        
        XCTAssertThrows([[PCAPISDK sharedInstance] getPlaylistContent:params withCompletionHandler:^(PCAPIQueryResponse *response) {}],@"Get playlist content API with nil access token");
    }
}

/*!
 * Test Case:PlaylistID is required to get playlist content response,otherwise test will fail.
 */

- (void)testForRequiredPlaylistID {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        XCTestExpectation *expectation = [self expectationWithDescription:@"Get playlist content API result"];
        
        PCGetPlaylistAPIParams *params = [[PCGetPlaylistAPIParams alloc] init];
        
        [[PCAPISDK sharedInstance] getPlaylist:params withCompletionHandler:^(NSArray *playlist, PCAPIQueryResponse *response) {
            
            switch (response.urlResponse.statusCode) {
                    
                case PCAPIResponseCodeOK: {

                    XCTAssertGreaterThanOrEqual([playlist count], 0);
                    
                    if (playlist.count) {
                        
                        PCPlaylist *object = [playlist objectAtIndex:0];
                        
                        PCGetPlaylistContentAPIParams* params = [[PCGetPlaylistContentAPIParams alloc] init];
                        if ([object.paths count]) {
                            params.itemUID = [[object.paths objectAtIndex:0] objectForKey:@"itemuid"];
                        } else {
                            XCTFail(@"Test Failed due to absence of playlist item ID");
                        }
                        
                        XCTAssertThrows([[PCAPISDK sharedInstance] getPlaylistContent:params withCompletionHandler:^(PCAPIQueryResponse *response) {}],@"Get playlist content API required playlist ID");
                    }
                    
                }
                    break;
                    
                default:
                    XCTFail(@"Test Failed due to absence of playlist");
                    break;
            }
            [expectation fulfill];
        }];
        [self waitForExpectationsWithTimeout:ExpectationTimeOut handler:^(NSError *error) {
            
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
            
        }];
        
    } else {
        
        XCTFail(@"Test failed due to nil access token");
        
    }
}

/*!
 * Test Case:ItemUid is required to get playlist content response,otherwise test will fail.
 */

- (void)testForRequiredItemUid {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        XCTestExpectation *expectation = [self expectationWithDescription:@"Get playlist content API result"];
        
        PCGetPlaylistAPIParams *params = [[PCGetPlaylistAPIParams alloc] init];
        
        [[PCAPISDK sharedInstance] getPlaylist:params withCompletionHandler:^(NSArray *playlist, PCAPIQueryResponse *response) {
            
            switch (response.urlResponse.statusCode) {
                    
                case PCAPIResponseCodeOK: {
                    
                    XCTAssertGreaterThanOrEqual([playlist count], 0);
                    
                    if (playlist.count) {
                        
                        PCPlaylist *object = [playlist objectAtIndex:0];
                        
                        PCGetPlaylistContentAPIParams* params = [[PCGetPlaylistContentAPIParams alloc] init];
                        params.playlistUID = object.UID;
                        
                        XCTAssertThrows([[PCAPISDK sharedInstance] getPlaylistContent:params withCompletionHandler:^(PCAPIQueryResponse *response) {
                            
                        }],@"Get playlist content API required Item UID");
                    }
                    
                }
                    break;
                    
                default:
                    XCTFail(@"Test Failed due to absence of playlist");
                    break;
            }
            [expectation fulfill];
        }];
        [self waitForExpectationsWithTimeout:ExpectationTimeOut handler:^(NSError *error) {
            
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
            
        }];
        
    } else {
        
        XCTFail(@"Test failed due to nil access token");
        
    }
}

/*!
 * Test Case: If response code is 200, get playlist content API will return the content of playlist item. In all other cases (error),it will not return playlist item content.
 */

- (void)testGetPlaylistContent {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        XCTestExpectation *expectation = [self expectationWithDescription:@"Get playlist content API result"];
        
        PCGetPlaylistAPIParams *params = [[PCGetPlaylistAPIParams alloc] init];
        
        [[PCAPISDK sharedInstance] getPlaylist:params withCompletionHandler:^(NSArray *playlist, PCAPIQueryResponse *response) {
            
            switch (response.urlResponse.statusCode) {
                    
                case PCAPIResponseCodeOK: {
                    
                    XCTAssertGreaterThanOrEqual([playlist count], 0);
                    
                    if (playlist.count) {
                        
                        PCPlaylist *object = [playlist objectAtIndex:0];
                        
                        PCGetPlaylistContentAPIParams* params = [[PCGetPlaylistContentAPIParams alloc] init];
                        params.playlistUID = object.UID;
                        if ([object.paths count]) {
                            params.itemUID = [[object.paths objectAtIndex:0] objectForKey:@"itemuid"];
                            [[PCAPISDK sharedInstance] getPlaylistContent:params withCompletionHandler:^(PCAPIQueryResponse *response) {
                                if (response.urlResponse.statusCode == PCAPIResponseCodeOK) {
                                    NSData *fileData = [NSData dataWithContentsOfURL:response.downloadedURL];
                                    XCTAssertNotNil(fileData,@"Get playlist content failed to download");
                                } else {
                                    XCTFail(@"Get playlist content API failed");
                                }
                                [expectation fulfill];
                            }];
                        } else {
                            XCTFail(@"Test Failed due to absence of playlist item ID");
                        }
                    }
                    
                }
                    break;
                    
                default:
                    XCTFail(@"Test Failed due to absence of playlist");
                    break;
            }
            [expectation fulfill];
        }];
        [self waitForExpectationsWithTimeout:ExpectationTimeOut*2 handler:^(NSError *error) {
            
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
            
        }];
        
    } else {
        
        XCTFail(@"Test failed due to nil access token");
        
    }
    
}

@end
