//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import <XCTest/XCTest.h>
#import "PCTestConstants.h"

@interface PCGetPlaylistItemsAPITests : XCTestCase

@end

@implementation PCGetPlaylistItemsAPITests

- (void)setUp {
    [super setUp];
    // Put setup code here. This method is called before the invocation of each test method in the class.
}

- (void)tearDown {
    // Put teardown code here. This method is called after the invocation of each test method in the class.
    [super tearDown];
}

/*!
 * Test Case: auth token is a required parameter. SDK should throw an exception in case auth token is nil.
 */
- (void)testNilAuthToken {
    // If User is not logged in, getPlaylistItems api should throw an exception
    if (![[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        PCGetPlaylistItemsAPIParams *params = [[PCGetPlaylistItemsAPIParams alloc] init];
        
        XCTAssertThrows([[PCAPISDK sharedInstance] getPlaylistItems:params withCompletionHandler:^(NSArray *playlistItems, PCAPIQueryResponse *response){}],@"Get playlist items API with nil access token");
    }
}


/*!
 * Test Case: Whenever PCGetPlaylistItemsAPIParams is allocated default value of start needs to be 1. Test fails otherwise.
 */
- (void)testDefaultParameterValue {
    
    PCGetPlaylistItemsAPIParams *param = [[PCGetPlaylistItemsAPIParams alloc] init];
    XCTAssertEqual(param.start, 1);
}

/*!
 * Test Case: If start is specified, count is required to get playlist items response, . otherwise test will fail.
 */

- (void)testManadatoryCountWhenStartSpecified {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        PCGetPlaylistItemsAPIParams *params = [[PCGetPlaylistItemsAPIParams alloc] init];
        params.start = 2;
        params.sort = @"asc";
        
        XCTAssertThrows([[PCAPISDK sharedInstance] getPlaylistItems:params withCompletionHandler:^(NSArray *playlistItems, PCAPIQueryResponse *response) {}],@"Get playlist items API throw exception without count");
        
    } else {
        
        XCTFail(@"Test Failed due to nil access token");
        
    }
    
}

/*!
 * Test Case:PlaylistUid is required to get playlist items response,otherwise test will fail.
 */

- (void)testRequiredPlaylistUid {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        PCGetPlaylistItemsAPIParams *params = [[PCGetPlaylistItemsAPIParams alloc] init];
        
        XCTAssertThrows([[PCAPISDK sharedInstance] getPlaylistItems:params withCompletionHandler:^(NSArray *playlistItems, PCAPIQueryResponse *response) {}],@"Get playlist items API throw exception without playlistUid");
        
    } else {
        
        XCTFail(@"Test Failed due to nil access token");
        
    }
    
}

/*!
 * Test Case: If response code is 200, Playlist items array contains zero or greater than zero value. Also playlist items object has to be of class PCResource type. In all other cases (error), playlist items has to be returned nil
 */
- (void)testToGetPlaylistItems {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        PCGetPlaylistAPIParams *params = [[PCGetPlaylistAPIParams alloc] init];
        XCTestExpectation *expectation = [self expectationWithDescription:@"Get playlist API Result"];
        
        [[PCAPISDK sharedInstance] getPlaylist:params withCompletionHandler:^(NSArray *playlist, PCAPIQueryResponse *response) {
            
            switch (response.urlResponse.statusCode) {
                    
                case PCAPIResponseCodeOK: {
                    
                    XCTAssertGreaterThanOrEqual([playlist count], 0);
                    if (playlist.count) {
                        
                        PCPlaylist *object = [playlist objectAtIndex:0];
                        PCGetPlaylistItemsAPIParams *params = [[PCGetPlaylistItemsAPIParams alloc] init];
                        params.playlistUID = object.UID;
                        
                        [[PCAPISDK sharedInstance]getPlaylistItems:params withCompletionHandler:^(NSArray *playlistItems, PCAPIQueryResponse *response) {
                            
                            switch (response.urlResponse.statusCode) {
                                case PCAPIResponseCodeOK: {
                                    
                                    XCTAssertGreaterThanOrEqual([playlistItems count], 0);
                                    if (playlistItems.count) {
                                        id object = [playlistItems objectAtIndex:0];
                                        NSLog(@"%@",NSStringFromClass([object class]));
                                        XCTAssertTrue([object isKindOfClass:[PCResource class]], @"Playlist are not PCResource class type");
                                    }
                                    
                                }
                                    break;
                                    
                                default:
                                    XCTAssertNil(playlistItems,@"Playlist items array return nil");
                                    break;
                            }
                            [expectation fulfill];
                        }];
                        
                    }
                }
                    break;
                    
                default:
                    XCTFail(@"Test could not be completed");
                    break;
            }
            [expectation fulfill];
        }];
        [self waitForExpectationsWithTimeout:ExpectationTimeOut handler:^(NSError *error) {
            
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
        }];
        
    } else {
        
        XCTFail(@"Test Failed due to nil access token");
        
    }
    
}


@end
