//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import <Foundation/Foundation.h>

/*!
 * Base class for email and telephone contact type.
 */
@interface PCContactType : NSObject

/*!
 * The type of contact either email or phone number
 */
@property (nonatomic, readonly) NSString *type;

/*!
 * The user's indx.
 */
@property (nonatomic, readonly) NSNumber *indx;

/*!
 * The user's preference.
 */
@property (nonatomic, readonly) NSNumber *preference;

@end


/*!
 * This class stores details of telephone contact type.
 */
@interface PCTelContactType : PCContactType

/*!
 * The user's unique contact number.
 */
@property (nonatomic ,readonly) NSNumber *number;

@end

/*!
 * This class stores details of email contact type.
 */
@interface PCEmailContactType : PCContactType

/*!
 * The user's email address.
 */
@property (nonatomic ,readonly) NSString *email;

@end

/*!
 * This class stores details of im(instant message) type.
 */
@interface PCInstantMessageType : PCContactType

/*!
 * The user's address.
 */
@property (nonatomic ,readonly) NSString *address;

@end

/*!
 * This class stores details of address.
 */
@interface PCAddressType : PCContactType

/*!
 * The user's city name.
 */
@property (nonatomic ,readonly) NSString *city;

/*!
 * The user's country name.
 */
@property (nonatomic ,readonly) NSString *country;

/*!
 * The user's pobox name.
 */
@property (nonatomic ,readonly) NSString *pobox;

/*!
 * The user's street name.
 */
@property (nonatomic ,readonly) NSString *street;

/*!
 * The user's zipcode.
 */
@property (nonatomic ,readonly) NSString *zipcode;

@end

/*!
 * PCContact stores all the details of a contact.
 */
@interface PCContact : NSObject

/*!
 * The user's version.
 */
@property (nonatomic, readonly) NSString *version;

/*!
 * The user's ID.
 */
@property (nonatomic, readonly) NSString *ID;

/*!
 * The user's nameprefix.
 */
@property (nonatomic, readonly) NSString *namePrefix;

/*!
 * The user's nickname.
 */
@property (nonatomic, readonly) NSString *nickname;

/*!
 * The user's first name.
 */
@property (nonatomic, readonly) NSString *firstName;

/*!
 * The user's last name.
 */
@property (nonatomic, readonly) NSString *lastName;

/*!
 * The user's note.
 */
@property (nonatomic, readonly) NSString *note;

/*!
 * The user's birthday.
 */
@property (nonatomic, readonly) NSString *birthday;

/*!
 * The user's anniversary.
 */
@property (nonatomic, readonly) NSString *anniversary;

/*!
 * The user's source.
 */
@property (nonatomic, readonly) NSString *source;

/*
 * Creation timestamp of the contact.
 */
@property (nonatomic, readonly) NSString *created;

/*
 * Modified timestamp of the contact.
 */
@property (nonatomic, readonly) NSString *modified;

/*!
 * The contact’s job title.
 */
@property (nonatomic, readonly) NSString *jobTitle;

/*!
 * The name of the organization associated with the contact.
 */
@property (nonatomic, readonly) NSString *orgName;

/*!
 * A bool to indicate if this contact can be reached in case of emergency.
 */
@property (nonatomic, readonly) BOOL incaseOfEmergency;

/*!
 * The user's photoReference.
 */
@property (nonatomic, readonly) NSString *photoReference;

/*!
 * The user's photo.
 */
@property (nonatomic, readonly) NSArray *photo;

/*!
 * A bool to indicate if this contact has been marked as favorite.
 */
@property (nonatomic, readonly) BOOL favorite;

/*!
 * An array of PZTelContactType objects.
 */
@property (nonatomic, readonly) NSArray *telphones;

/*!
 * An array of  PCEmailContactType objects.
 */
@property (nonatomic, readonly) NSArray *emails;

/*!
 * An array of  PCAddressType objects.
 */
@property (nonatomic, readonly) NSArray *addresses;

/*!
 * An array of  PCInstantMessageType objects.
 */
@property (nonatomic, readonly) NSArray *imArray;

@end