//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.


#import <Foundation/Foundation.h>
#import <CoreData/CoreData.h>

/*!
 * This class stores the metadata information of a file or a folder.
 */
@interface PCResource : NSObject

/*!
 * Maps to the ID3 album field for MP3 files.
 */
@property (nonatomic, readonly, copy) NSString *album;

/*!
 * Maps to the ID3 artist field for MP3 files.
 */
@property (nonatomic, readonly, copy) NSString *artist;

/*!
 * Maps to the file's EXIF Date and Time tag.
 */
@property (nonatomic, readonly, strong) NSDate *capturedDate;

/*!
 * The file content's SHA-256 checksum value.
 */
@property (nonatomic, readonly, copy) NSString *checkSum;

/*!
 * A unique string representing the file. Will change if file content is updated. Required for certain APIs, such as /thumbnails.
 */
@property (nonatomic, readonly, copy) NSString *contentToken;

/*!
 * contentType of the file. image/png for png files etc.
 */
@property (nonatomic, readonly, copy) NSString *contentType;

/*!
 * The number of items the folder contains.
 */
@property (nonatomic, readonly, strong) NSNumber *count;

/*!
 * The date, in UTC format, that the file was created on the client system. For example, where a desktop client uploads a photo originally taken on a camera, Cre- ation-Date will differ from Capture-Date, while on a camera phone the two attributes are likely to match.
 */
@property (nonatomic, readonly, strong) NSDate *creationDate;

/*!
 * If true, the file has been logically deleted.
 */
@property (nonatomic, readonly, strong) NSNumber *deleted;

/*!
 * The duration of an audio file.
 */
@property (nonatomic, readonly, strong) NSNumber *duration;

/*!
 * The file's extension, as extracted from the filename.
 */
@property (nonatomic, readonly, copy) NSString *extension;

/*!
 * If true, the resource is a file.
 */
@property (nonatomic, readonly, strong) NSNumber *file;

/*!
 * The full path to file/folder.
 */
@property (nonatomic, readonly, copy) NSString *fullPath;

/*!
 * Maps to the ID3 genre field for MP3 files.
 */
@property (nonatomic, readonly, copy) NSString *genre;

/*!
 * Maps to the EXIF GPS Latitude tag for image files.
 */
@property (nonatomic, readonly, strong) NSNumber *geoLatitude;

/*!
 * Maps to the EXIF GPS Longitude tag for image files.
 */
@property (nonatomic, readonly, strong) NSNumber *geoLongitude;

/*!
 * The height of an image in pixels.
 */
@property (nonatomic, readonly, strong) NSNumber *height;

/*!
 * The unique ID to distinguis an item in a playlist.
 */
@property (nonatomic, readonly, copy) NSString *itemUID;

/*!
 * The location country.
 */
@property (nonatomic, readonly, copy) NSString *locationCountry;

/*!
 * The city, town, village or general locality.
 */
@property (nonatomic, readonly, copy) NSString *locationLocality;

/*!
 * The state, province or general region.
 */
@property (nonatomic, readonly, copy) NSString *locationRegion;

/*!
 * Specifies the content type of the file.
 */
@property (nonatomic, readonly, copy) NSString *mimeType;

/*!
 * The name of the file.
 */
@property (nonatomic, readonly, copy) NSString *name;

/*!
 * The folder where the file is located.
 */
@property (nonatomic, readonly, copy) NSString *parentPath;

/*!
 * Unique identifier of the resource.
 */
@property (nonatomic, readonly, copy) NSString *uri;

/*!
 * Tags of the resource.
 */
@property (nonatomic, readonly, copy) NSString *tags;

/*!
 * The size of the file, in bytes.
 */
@property (nonatomic, readonly, strong) NSNumber *size;

/*!
 * Dictates where a media file should appear in a time- line view on a client application.
 */
@property (nonatomic, readonly, strong) NSDate *timelineDate;

/*!
 * Maps to the ID3 title field for MP3 files.
 */
@property (nonatomic, readonly, copy) NSString *title;

/*!
 * File version. This number increases when the file is updated.
 */
@property (nonatomic, readonly, strong) NSNumber *version;

/*!
 * Creation timestamp for the file's current version, in ISO-8601 format.
 */
@property (nonatomic, readonly, strong) NSDate *versionCreated;

/*!
 * The width of an image in pixels.
 */
@property (nonatomic, readonly, strong) NSNumber *width;

/*!
 * Check whether virtual folder is immutable.
 */
@property (nonatomic, readonly, assign) BOOL immutable;

/*!
 * Call this methods to create a PCResource using a managedObjectID. managedObjectID is mapped with coredata object and a PCResource is created out of the values copied from the coredata object.
 */
- (id)initWithDBResourceID:(NSManagedObjectID *)managedObjectID;

@end
