//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import <Foundation/Foundation.h>

/*!
 * PCLink stores all the definition of a link.
 */
@interface PCLink : NSObject

/*!
 * Can be one of 'thumbnail', 'next', 'prev', etc.
 */
@property (nonatomic, readonly) NSString *rel;

/*!
 * The URI of the link.
 */
@property (nonatomic, readonly) NSString *link;

@end


/*!
 * PCUser stores all the defination of a share user.
 */
@interface PCUser : NSObject

/*!
 * The profile identifier. This is only accessible to the owner of the share; the sharer of the share or a member that is associated with the owner.
 */
@property (nonatomic, readonly) NSString *UID;

@end



/*!
 * PCShareResource stores all the defination of a share resource.
 */
@interface PCShareResource : NSObject

/*!
 *  The resource identifier.
 */
@property (nonatomic, readonly) NSString *uid;

/*!
 *  Checksum of the resource.
 */
@property (nonatomic, readonly) NSString *checksum;

/*!
 *  ContentToken of the resource.
 */
@property (nonatomic, readonly) NSString *contentToken;

/*!
 *  LastModifiedDate of the resource.
 */
@property (nonatomic, readonly) NSDate *lastModifiedDate;

/*!
 *  Location of the resource.
 */
@property (nonatomic, readonly) NSString *location;

/*!
 *  Name of the resource.
 */
@property (nonatomic, readonly) NSString *name;

/*!
 *  Id of the resource.
 */
@property (nonatomic, readonly) NSString *resourceId;

/*!
 *  Size of the resource.
 */
@property (nonatomic, readonly) NSNumber *size;

/*!
 *  Type of the resource.
 */
@property (nonatomic, readonly) NSString *type;

/*!
 *  Subtype of the resource.
 */
@property (nonatomic, readonly) NSString *subType;

@end



/*!
 * PCShareItem stores all the defination of a Share.
 */
@interface PCShareItem : NSObject

/*!
 * The Share links for the share.
 */
@property (nonatomic,readonly) PCLink *link;

/*!
 * The unique identifier for the share.
 */
@property (nonatomic, readonly) NSString *UID;

/*!
 * The name of the share. On share creation,this can be a user-defined name.
 */
@property (nonatomic, readonly) NSString *name;

/*!
 * The total count of shared resources.
 */
@property (nonatomic, readonly, strong) NSNumber *totalResourceCount;

/*!
 * The owner of the share.
 */
@property (nonatomic, readonly) PCUser *owner;

/*!
 * The sharer of the share.
 */
@property (nonatomic, readonly) PCUser *sharer;

/*!
 * The list of resource share. Contains object of PCShareResource type.
 */
@property (nonatomic, readonly) NSArray *resources;

/*!
 * The time to live of the share in seconds. Clients can specify either this or 'expirationDate' when
   creating a share, but not both. If specified, the value must be within a configurable min and max range.
   Other- wise, the time to live of the share is set to a configurable default value. When accessing a share,
   this returns the remaining time to live on the share. A value of 0 indicates that the share does not expire.
 */
@property (nonatomic, readonly) NSNumber *expiresIn;

/*!
 * The date and time the share expires. Clients can specify either this or 'expiresIn' when creating a share,
   but not both. If specified, the expiration interval must be within a configurable min and max range.
   Otherwise, the time to live of the share is set to a configurable default value. When accessing a share,
   this returns the date and time the share expires. If the share does not expire, this field is not populated.
 */
@property (nonatomic, readonly) NSDate *expirationDate;

/*!
 * The date and time the share was created.
 */
@property (nonatomic, readonly) NSDate *creationDate;

/*!
 * The date and time the share was last modified from the perspective of the owner. This value is updated by
   the following events: resource added, updated or removed; member added, updated or removed; member
   accepts, declines or leaves; share name or visibility updated; or permissions updated.
 */
@property (nonatomic, readonly) NSDate *lastModifiedDate;

/*!
 * The date and time the share was last modified from the perspective of a member. This value is only updated
   when a resource is added or updated. or modified.
 */
@property (nonatomic, readonly) NSDate *inboundLastModifiedDate;

/*!
 * The invite key of a public share.
 */
@property (nonatomic, readonly) NSString *publicInviteKey;

@end



/*!
 * PCShare stores all the defination of a user shares.
 */
@interface PCShare : NSObject

/*!
 * Resource links. An array of PCLink object.
 */
@property (nonatomic, readonly) NSArray *links;

/*!
 * An array of PCShareItem object.
 */
@property (nonatomic, readonly) NSArray *shareList;

/*!
 * Total count of the share.
 */
@property (nonatomic, readonly) NSNumber *count;

@end
