//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import "UIViewController+ResourceActions.h"
#import "UIViewController+Alert.h"
#ifdef HUBBLE_MEDIA_BROWSER
#import "PCResource+Info.h"
#import "MoveToPlaylistViewController.h"
#import "PCGetTagsAPIParams.h"
#import "PCSetTagsAPIParams.h"
#import "PCDeleteTagsAPIParams.h"
#endif
#ifdef KEPLER_FILE_BROWSER
#import "MetaDataViewController.h"
#import "MoveViewController.h"
#import "CopyViewController.h"
#import "PCResource+Info.h"
#import "NSString+Addition.h"
#import "MedialistViewController.h"
#import "MoveCopyBaseViewController.h"
#endif


@implementation UIViewController (ResourceActions)


- (NSString *)stringForResourceAction:(ResourceAction)action {

    switch (action) {
#ifdef KEPLER_FILE_BROWSER

        case copyResourceAction:
            return NSLocalizedString(@"Copy", nil);
            break;
        case moveResourceAction:
            return NSLocalizedString(@"Move", nil);
            break;
        case renameResourceAction:
            return NSLocalizedString(@"Rename", nil);
            break;
        case deleteResourceAction:
            return NSLocalizedString(@"delete", nil);
        case restoreResourceAction:
            return NSLocalizedString(@"Restore", nil);
            break;
        case renameVFolderResourceAction:
            return NSLocalizedString(@"Rename virtual folder", nil);
            break;
        case deleteVFolderResourceAction:
            return NSLocalizedString(@"Delete virtual folder", nil);
            break;
#endif
#ifdef HUBBLE_MEDIA_BROWSER
        case addToAlbumResourceAction:
            return NSLocalizedString(@"Add to Album", nil);
            break;
        case addToPlaylistResourceAction:
            return NSLocalizedString(@"Add to Playlist", nil);
            break;
        case removeFromAlbumResourceAction:
            return NSLocalizedString(@"Remove from Album", nil);
            break;
        case removeFromPlaylistResourceAction:
            return NSLocalizedString(@"Remove from Playlist", nil);
            break;
#endif
        case addToGalleryResourceAction:
            return NSLocalizedString(@"Add to gallery", nil);
            break;
        default:
            return nil;
            break;
    }
}

- (void)didBeginDoingResourceAction:(ResourceAction)action {
    // abstract method
}

- (void)didFinishDoingAction:(ResourceAction)action onResource:(id)resource result:(id)result response:(id)response {
    // abstract method
}

- (id)dataSourceForResourceAction:(ResourceAction)action {
    // abstract method
    return nil;
}

#pragma mark -
#pragma mark Cancel
- (void)addCancelAction:(UIAlertController *)alertController {
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Cancel", nil)
                    style:UIAlertActionStyleCancel
                andAction:nil];
}

#ifdef KEPLER_FILE_BROWSER

#pragma mark -
#pragma mark Rename

- (void)showRenameAlert:(UIAlertController *)alertController forSelectedResource:(PCResource *)selectedResource {
    
    __weak UIViewController *blocksafeSelf = self;
    
    alertController = [UIAlertController alertControllerWithTitle:NSLocalizedString(@"Enter a new name", nil)
                                                          message:nil
                                                   preferredStyle:UIAlertControllerStyleAlert];

    [self alertController:alertController showTextInputAlertWithTitle:selectedResource.name andOKCompletionHandler:^(UIAlertAction *action) {
        UITextField *textField = [alertController.textFields firstObject];
        if (textField.text.length) {
            
            [blocksafeSelf didBeginDoingResourceAction:renameResourceAction];
            // Intialize the PCFOPSAPIParams object.
            PCFOPSAPIParams *rename = [[PCFOPSAPIParams alloc] init];
            rename.source = selectedResource.fullPath;
            rename.target = [NSString stringWithFormat:@"%@/%@", selectedResource.parentPath, textField.text];
            
            if ([blocksafeSelf isKindOfClass:[MedialistViewController class]]) {
                PCResourceItem *resourceItem = (PCResourceItem *)selectedResource;
                
                /*!
                 @Description-This method name is renameItem,Call this method to rename a file/folder.
                 @ Parameters - (PCFOPSAPIParams object) attributes such as source and target of the resource.
                 @ReturnType- PCAPI object This return type we can cancel the ongoing request .
                 @CompletionHandler-This completion block gets called on the completion of a rename operation. meta data of renamed folder is returned in completion block, Return type are (PCAPIQueryResponse object)andany type respective selected source of resourceItem (PCPhoto,PCVideo,PCAudio,PCFile).
                 */
                
                [resourceItem renameItem:rename withCompletionHandler:^(id resourceItem, PCAPIQueryResponse *response) {
                    [blocksafeSelf didFinishDoingAction:renameResourceAction onResource:(PCResourceItem *)selectedResource result:resourceItem response:response];
                }];
            }
            else{
                /*
                 @Description :
                 This API is used to rename a file/folder. Once the rename process is succeed/failed result will come with completion block.
                 
                 @Parameters :
                 PCFOPSAPIParams:
                 * This class stores all the parameters required to rename a file/folder. It contains attributes such as source, target, safe, conflictsolve and overrideRule.
                 * For providing access token to SDK user have to logged In.
                 * source and target are required parameter to call this API.
                 * conflictsolve, safe and overrideRule are optional parameter to call this API.
                 * If user will not provide required parameter, API will throw an NSInvalidArgumentException and user have to handle it.
                 * More information about parameters you will find in PCFOPSAPIParams.h class.
                 
                 @CompletionHandler :
                 This completion block gets called once the rename process is succeed/failed. completion block contains below parameter.
                 PCResource : This object stores metadata information of a file or a folder.
                 PCAPIQueryResponse : Stores all the server response and error for this API.
                 
                 @To rename a file and folder, you must take the following steps:
                 * Include VerizonCloudAPI.framework header file in your class.
                 #import <VerizonCloudAPI/VerizonCloudAPI.h>
                 * Initalize the PCFOPSAPIParams class and provide the required parameter.
                 * Initalize the PCAPISDK object and call the  rename:withCompletionHandler: API.
                 * Perform the  rename:withCompletionHandler: API call using the code sample below.
                 * Get the result (succeed or failed) in completion block.
                 * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
                 */
                // Initilize the PCAPISDK and call the API.
                
            [[PCAPISDK sharedInstance] rename:rename withCompletionHandler:^(PCResource *metadata, PCAPIQueryResponse *response) {
                // Get the result in this block.
                [blocksafeSelf didFinishDoingAction:renameResourceAction onResource:selectedResource result:metadata response:response];
            }];
            }
        }
    }];
}

- (void)addRenameAction:(UIAlertController *)alertController forSelectedResource:(PCResource *)selectedResource {
    __weak UIViewController *blocksafeSelf = self;
    
    __block UIAlertController *renameAlertController = nil;
    
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Rename", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    
                    [blocksafeSelf showRenameAlert:renameAlertController forSelectedResource:selectedResource];
                }];
}
#pragma mark -
#pragma mark Restore

- (void)addRestoreAction:(UIAlertController *)alertController forSelectedResource:(PCResource *)selectedResource {
    __weak UIViewController *blocksafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Restore", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    
                    NSString *message = NSLocalizedString(@"Are you sure want to restore folder", nil);
                    
                    if (selectedResource.fileType == imageFileType || selectedResource.fileType == musicFileType ||  selectedResource.fileType == videoFileType || selectedResource.fileType == documentFileType) {
                        
                        message = NSLocalizedString(@"Are you sure want to restore file", nil);
                        
                    }
                    
                    [blocksafeSelf showOKCancelAlertWithTitle:NSLocalizedString(@"Alert", nil)
                                          andMessage:message
                                            okAction:^(UIAlertAction *action) {
                                                
                                                /*
                                                 @Description -
                                                 This API is used to restore files or folders that are marked as deleted. Once the restore process is succeed/failed result will come with completion block.
                                                 
                                                 @Parameters -
                                                 PCRestoreAPIParams :
                                                 * This class stores all the parameters required to restore files or folders that are marked as deleted.
                                                 * paths and accessToken are required parameter to call this API. For providing access token to SDK user have to logged In.
                                                 * If user will not provide required parameter, API will throw an NSInvalidArgumentException and user have to handle it.
                                                 * There is no optional parameter for this API.
                                                 * More information about parameters you will find in PCRestoreAPIParams.h class.
                                                 
                                                 @CompletionHandler :
                                                 This completion block gets called once the restore process is succeed/failed. completion block contains below parameter.
                                                 PCAPIQueryResponse :
                                                 * Stores all the server response and error for this API.
                                                 
                                                 @To restore files or folders that are marked as deleted, you must take the following steps:
                                                 * Include VerizonCloudAPI.framework header file in your class.
                                                    #import <VerizonCloudAPI/VerizonCloudAPI.h>
                                                 * Initalize the PCRestoreAPIParams class and provide the required parameter.
                                                 * Initalize the PCAPISDK object and call the restore:withCompletionHandler: API.
                                                 * Perform the restore:withCompletionHandler: API call using the code sample below.
                                                 * Get the result (succeed or failed) in completion block.
                                                 * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
                                                 */

                                            // Initialize the PCRestoreAPIParams class.
                                            PCRestoreAPIParams *restoreParams = [[PCRestoreAPIParams alloc] init];
                                                
                                            restoreParams.paths = @[selectedResource.fullPath]; // provide the required parameter.
                                        
                                            [blocksafeSelf didBeginDoingResourceAction:restoreResourceAction];

                                            // Initialize PCAPISDK and call restore:withCompletionHandler API.
                                            [[PCAPISDK sharedInstance] restore:restoreParams withCompletionHandler:^(PCAPIQueryResponse *response) {
                                                        [blocksafeSelf didFinishDoingAction:restoreResourceAction onResource:selectedResource result:nil response:response];
                                            }];
                                            }
                                        cancelAction:^(UIAlertAction *action) {
                                            
                                        }];
                    
    }];

}
#pragma mark -
#pragma mark Delete

- (void)addDeleteAction:(UIAlertController *)alertController forSelectedResource:(PCResource *)selectedResource {
    __weak UIViewController *blocksafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Delete", nil)
                    style:UIAlertActionStyleDestructive
                andAction:^(UIAlertAction * _Nonnull action) {
            
                    NSString *message = NSLocalizedString(@"Are you sure want to delete folder", nil);
                    
                    if (selectedResource.fileType == imageFileType || selectedResource.fileType == musicFileType ||  selectedResource.fileType == videoFileType || selectedResource.fileType == documentFileType) {
                        
                        message = NSLocalizedString(@"Are you sure want to delete file", nil);
                        
                    }
                    
                    [blocksafeSelf showOKCancelAlertWithTitle:NSLocalizedString(@"Warning", nil)
                                                   andMessage:message
                                                     okAction:^(UIAlertAction *action) {
                                                         // Intialize the PCCreateFolderAPIParams object.
                                                         PCDeleteParams *params = [[PCDeleteParams alloc] init];
                                                         params.path = selectedResource.fullPath;
                                                         
                                                         
                                                         [blocksafeSelf didBeginDoingResourceAction:deleteResourceAction];
                                                         
                                                         if ([blocksafeSelf isKindOfClass:[MedialistViewController class]]) {
                                                             PCResourceItem *resourceItem = (PCResourceItem *) selectedResource;
                                                             [resourceItem deleteItemWithCompletionHandler:^(PCAPIQueryResponse *response) {
                                                                 [blocksafeSelf didFinishDoingAction:deleteResourceAction onResource:selectedResource result:nil response:response];
                                                             }];
                                                         }
                                                         else{
                                                             /*
                                                              @Description :
                                                              This API is used to delete a file/folder. Once the delete process is succeed/failed result will come with completion block.
                                                              
                                                              @Parameters :
                                                              PCDeleteParams:
                                                              * This class stores all the parameters required to delete a file/folder. It contains attributes such as path and purge.
                                                              * For providing access token to SDK user have to logged In.
                                                              * path is required parameter to call this API.
                                                              * purge is optional parameter to call this API.
                                                              * If user will not provide required parameter, API will throw an NSInvalidArgumentException and user have to handle it.
                                                              * More information about parameters you will find in PCDeleteParams.h class.
                                                              
                                                              @CompletionHandler :
                                                              This completion block gets called once the delete process is succeed/failed. completion block contains below parameter.
                                                              PCAPIQueryResponse : Stores all the server response and error for this API.
                                                              
                                                              @To delete a file and folder, you must take the following steps:
                                                              * Include VerizonCloudAPI.framework header file in your class.
                                                               #import <VerizonCloudAPI/VerizonCloudAPI.h>
                                                              * Initalize the PCDeleteParams class and provide the required parameter.
                                                              * Initalize the PCAPISDK object and call the  deletePath:withCompletionHandler: API.
                                                              * Perform the  deletePath:withCompletionHandler: API call using the code sample below.
                                                              * Get the result (succeed or failed) in completion block.
                                                              * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
                                                              */

                                                             // Initilize the PCAPISDK and call the API.
                                                             [[PCAPISDK sharedInstance] deletePath:params withCompletionHandler:^(PCAPIQueryResponse *response) {
                                                                 // Get the result in this block.
                                                                 [blocksafeSelf didFinishDoingAction:deleteResourceAction onResource:selectedResource result:nil response:response];
                                                             }];

                                                         }
                                                         
                                                     }
                                                 cancelAction:^(UIAlertAction *action) {
                                                     
                                                 }];
                }];
}

#pragma mark -
#pragma mark Move

- (void)addMoveAction:(UIAlertController *)alertController forSelectedResource:(PCResource *)selectedResource {
    __weak UIViewController *blocksafeSelf = self;
    
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Move", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    
                    UIStoryboard *storyBoard = [UIStoryboard storyboardWithName:@"Main" bundle:nil];
                    MoveViewController *controller = (MoveViewController*)[storyBoard instantiateViewControllerWithIdentifier: @"MoveViewController"];
                    controller.source = selectedResource.fullPath;
                    if ([blocksafeSelf isKindOfClass:[MedialistViewController class]]) {
                        controller.tabType = mediaTabType;
                    }
                    else{
                        controller.tabType = rootTabType;
                    }
                   
                    [controller setCompletionHandler:^(PCResource *metadata, PCAPIQueryResponse *response) {

                        [blocksafeSelf didFinishDoingAction:moveResourceAction onResource:selectedResource result:metadata response:response];
                    }];
                   
                    UINavigationController *navController = [[UINavigationController alloc] initWithRootViewController:controller];
                    [blocksafeSelf presentViewController:navController animated:YES completion:nil];
                }];
}

#pragma mark -
#pragma mark Copy

- (void)addCopyAction:(UIAlertController *)alertController forSelectedResource:(PCResource *)selectedResource {
    __weak UIViewController *blocksafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Copy", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    
                    UIStoryboard *storyBoard = [UIStoryboard storyboardWithName:@"Main" bundle:nil];
                    CopyViewController *controller = (CopyViewController*)[storyBoard instantiateViewControllerWithIdentifier: @"CopyViewController"];
                    controller.source = selectedResource.fullPath;
                    if ([blocksafeSelf isKindOfClass:[MedialistViewController class]]) {
                        controller.tabType = mediaTabType;
                    }
                    else{
                        controller.tabType = rootTabType;
                    }

                    [controller setCompletionHandler:^(PCResource *metadata, PCAPIQueryResponse *response){
                        
                        [blocksafeSelf didFinishDoingAction:copyResourceAction onResource:selectedResource result:metadata response:response];
                    }];
                    
                    UINavigationController *navController = [[UINavigationController alloc] initWithRootViewController:controller];
                    [blocksafeSelf presentViewController:navController animated:YES completion:nil];
                }];
}

#pragma mark -
#pragma mark rename virtual folder.

-(void) addRenameVirtualFolderAction:(UIAlertController *)alertController forSelectedResource:(PCResource *)selectedResource {
    __weak UIViewController *blocksafeSelf = self;
    
    __block UIAlertController *renameVFolderAlertController = nil;
    
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Rename", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    
                    [blocksafeSelf showRenameVirtualFolderAlert:renameVFolderAlertController forSelectedResource:selectedResource];
                }];
}

-(void) showRenameVirtualFolderAlert:(UIAlertController *)alertController forSelectedResource:(PCResource *)selectedResource {
    __weak UIViewController *blocksafeSelf = self;
    
    alertController = [UIAlertController alertControllerWithTitle:NSLocalizedString(@"Enter a new name", nil) message:nil preferredStyle:UIAlertControllerStyleAlert];
    
    [self alertController:alertController showTextInputAlertWithTitle:selectedResource.name andOKCompletionHandler:^(UIAlertAction *action) {
        UITextField *textField = [alertController.textFields firstObject];
        if (textField.text.length) {
            [blocksafeSelf didBeginDoingResourceAction:renameVFolderResourceAction];
            
            /*
             @Description :
             This API is used to rename a virtual folder. Once the rename process is succeed/failed result will come with completion block.
             
             @Parameters :
             PCRenameVFolderAPIParams:
             * This class stores all the parameters required to rename a virtual folder. It contains attributes such as src and target.
             * For providing access token to SDK user have to logged In.
             * src and target are required parameter to call this API.
             * If user will not provide required parameter, API will throw an NSInvalidArgumentException and user have to handle it.
             * More information about parameters you will find in PCRenameVFolderAPIParams.h class.
             
             @CompletionHandler :
             This completion block gets called once the rename process is succeed/failed. completion block contains below parameter.
             PCResource : This object stores metadata information of a virtual folder.
             PCAPIQueryResponse : Stores all the server response and error for this API.
             
             @To rename a virtual folder, you must take the following steps:
             * Include VerizonCloudAPI.framework header file in your class.
             #import <VerizonCloudAPI/VerizonCloudAPI.h>
             * Initalize the PCRenameVFolderAPIParams class and provide the required parameter.
             * Initalize the PCAPISDK object and call the  renameVirtualFolder:withCompletionHandler: API.
             * Perform the  renameVirtualFolder:withCompletionHandler: API call using the code sample below.
             * Get the result (succeed or failed) in completion block.
             * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
             */
            
            // Intialize the PCRenameVFolderAPIParams object
            PCRenameVFolderAPIParams *params = [[PCRenameVFolderAPIParams alloc] init];
            params.src = selectedResource.name;
            params.target = textField.text;
            
            // Initilize the PCAPISDK and call the API.
            [[PCAPISDK sharedInstance] renameVirtualFolder:params withCompletionHandler:^(PCResource *metadata, PCAPIQueryResponse *response) {
                
                [blocksafeSelf didFinishDoingAction:renameVFolderResourceAction onResource:selectedResource result:metadata response:response];
            }];
            
            
        }
    }];
}

#pragma mark -
#pragma mark Delete Virtual Folder

- (void)addDeleteVFolderAction:(UIAlertController *)alertController forSelectedResource:(PCResource *)selectedResource {
    __weak UIViewController *blocksafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Delete", nil)
                    style:UIAlertActionStyleDestructive
                andAction:^(UIAlertAction * _Nonnull action) {
                    
                    NSString *message = NSLocalizedString(@"Are you sure want to delete virtual folder", nil);
                    
                    [blocksafeSelf showOKCancelAlertWithTitle:NSLocalizedString(@"Warning", nil)
                                                   andMessage:message
                                                     okAction:^(UIAlertAction *action) {
                                                         // Intialize the PCDeleteVFolderAPIParams object.
                                                         PCDeleteVFolderAPIParams *params = [[PCDeleteVFolderAPIParams alloc] init];
                                                         params.folderName = selectedResource.name;
                                                         params.force = YES;
                                                         
                                                        [blocksafeSelf didBeginDoingResourceAction:deleteVFolderResourceAction];
                                                         
                                                         /*
                                                          @Description :
                                                          This API is used to delete a virtual folder. Once the delete process is succeed/failed result will come with completion block.
                                                          
                                                          @Parameters :
                                                          PCDeleteVFolderAPIParams:
                                                          * This class stores all the parameters required to delete a virtual folder. It contains attributes such as folderName and force.
                                                          * For providing access token to SDK user have to logged In.
                                                          * folderName is required parameter to call this API.
                                                          * force is optional parameter to call this API. Default is false.
                                                          * If user will not provide required parameter, API will throw an NSInvalidArgumentException and user have to handle it.
                                                          * More information about parameters you will find in PCDeleteVFolderAPIParams.h class.
                                                          
                                                          @CompletionHandler :
                                                          This completion block gets called once the delete process is succeed/failed. completion block contains below parameter.
                                                          PCAPIQueryResponse : Stores all the server response and error for this API.
                                                          
                                                          @To delete a virtual folder, you must take the following steps:
                                                          * Include VerizonCloudAPI.framework header file in your class.
                                                          #import <VerizonCloudAPI/VerizonCloudAPI.h>
                                                          * Initalize the PCDeleteVFolderAPIParams class and provide the required parameter.
                                                          * Initalize the PCAPISDK object and call the  deleteVirtualFolder:withCompletionHandler: API.
                                                          * Perform the  deleteVirtualFolder:withCompletionHandler: API call using the code sample below.
                                                          * Get the result (succeed or failed) in completion block.
                                                          * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
                                                          */
                                                         
                                                             // Initilize the PCAPISDK and call the API.
                                                         
                                                         [[PCAPISDK sharedInstance] deleteVirtualFolder:params withCompletionHandler:^(PCAPIQueryResponse *response) {
                                                             [blocksafeSelf didFinishDoingAction:deleteVFolderResourceAction onResource:selectedResource result:nil response:response];
                                                         }];
                                                         
                                                             
                                                         
                                                     }
                                                 cancelAction:^(UIAlertAction *action) {
                                                     
                                                 }];
                }];
}

#endif

#ifdef HUBBLE_MEDIA_BROWSER
#pragma mark -
#pragma mark Add file
- (void)addFileInfoAction:(UIAlertController *)alertController forSelectedResource:(PCResource *)selectedResource {
    __weak UIViewController *blocksafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"File Info", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    
                    [blocksafeSelf didBeginDoingResourceAction:fileInfoResourceAction];
                    
                    /*
                     @Description -
                     This API is used to fetch the tags for the file or folder under the specified path. Once the fetch process is done/failed result will come with completion block.
                     
                     @Parameters -
                     PCGetTagsAPIParams : This class store all the parameters required fetch the tags for the file or folder under the specified path.
                     * path and accessToken are required parameter to call this API.
                     * For providing access token to SDK user have to logged In.
                     * If user will not pass required parameter, API will throw an NSInvalidArgumentException and user have to handle it.
                     * More information about parameters you will find in PCGetTagsAPIParams.h class.
                     
                     @CompletionHandler -
                     This completion block gets called once fetch tags process is done/failed. completion block contains below parameter.
                     PCTags : PCTags stores all the definition of a tags.
                     PCAPIQueryResponse : Stores all the server response and error for this API.
                     
                     @To fetch the tags, you must take the following steps: -
                     * Include VerizonCloudAPI.framework header file in your class.
                     #import <VerizonCloudAPI/VerizonCloudAPI.h>
                     * Initalize the PCGetTagsAPIParams class and provide the required parameter.
                     * Initalize the PCAPISDK object and call the getTags:withCompletionHandler: API.
                     * Get the result(Success or failed) in completion block.
                     * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
                     */
                    
                    // Sample code for getTags:withCompletionHandler: API

                    
                    PCGetTagsAPIParams *params = [[PCGetTagsAPIParams alloc] init];
                    params.path = selectedResource.fullPath;
                    
                    [[PCAPISDK sharedInstance] getTags:params withCompletionHandler:^(PCTags *tag, PCAPIQueryResponse *response) {
                        if (response.urlResponse.statusCode == PCAPIResponseCodeOK || response.urlResponse.statusCode ==  PCAPIResponseCodeNoContent) {
                            [blocksafeSelf showOKAlertWithTitle:NSLocalizedString(@"File Info", nil)
                                                     andMessage:nil
                                               attributedString:[selectedResource fileInfoWithTags:tag]
                                                       okAction:nil];
                        }
                        [blocksafeSelf didFinishDoingAction:fileInfoResourceAction onResource:selectedResource result:tag response:response];
                    }];
    }];
                    
                    
}
#pragma mark -
#pragma mark set tags
- (void)addSetTagsAction:(UIAlertController *)alertController forSelectedResource:(PCResource *)selectedResource {
    __weak UIViewController *blocksafeSelf = self;
    
    __block UIAlertController *setTagsAlertController = nil;
    
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Set Tag", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    [blocksafeSelf showSetTagsAlert:setTagsAlertController forSelectedResource:selectedResource];
                }];
}

- (void)showSetTagsAlert:(UIAlertController *)alertController forSelectedResource:(PCResource *)selectedResource {
    
    __weak UIViewController *blocksafeSelf = self;
    
    alertController = [UIAlertController alertControllerWithTitle:NSLocalizedString(@"Set a new tag", nil)
                                                          message:nil
                                                   preferredStyle:UIAlertControllerStyleAlert];
    
    [self alertController:alertController showTextInputAlertWithTitle:nil andOKCompletionHandler:^(UIAlertAction *action) {
        UITextField *textField = [alertController.textFields firstObject];
        
        if (textField.text.length) {
            
            [blocksafeSelf didBeginDoingResourceAction:setTagsResourceAction];
            
            /*
             @Description -
             This API is used to update the tags for the file or folder under the specified path. Once the update process is done/failed result will come with completion block.
             
             @Parameters -
             PCSetTagsAPIParams : This class store all the parameters required to update the tags for the file or folder under the specified path.
             * uri, tags and accessToken are required parameter to call this API.
             * For providing access token to SDK user have to logged In. 
             * If user will not pass required parameter, API will throw an NSInvalidArgumentException and user have to handle it.
             * More information about parameters you will find in PCSetTagsAPIParams.h class.
             
             @CompletionHandler -
             This completion block gets called once update tags process is done/failed. completion block contains below parameter.
             PCTags : PCTags stores all the definition of a tags.
             PCAPIQueryResponse : Stores all the server response and error for this API.
             
             @To update the tags, you must take the following steps: -
             * Include VerizonCloudAPI.framework header file in your class.
                #import <VerizonCloudAPI/VerizonCloudAPI.h>
             * Initalize the PCSetTagsAPIParams class and provide the required parameter.
             * Initalize the PCAPISDK object and call the setTags:withCompletionHandler: API.
             * Get the result(Success or failed) in completion block.
             * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
             */
            
            // Sample code for setTags:withCompletionHandler: API
            
            PCSetTagsAPIParams *params = [[PCSetTagsAPIParams alloc] init];
            params.uri = selectedResource.uri;
            params.tags = textField.text;
            
            [[PCAPISDK sharedInstance] setTags:params withCompletionHandler:^(PCTags *tag, PCAPIQueryResponse *response) {
                 [blocksafeSelf didFinishDoingAction:setTagsResourceAction onResource:selectedResource result:tag response:response];
            }];
        }
    }];
    
}
#pragma mark -
#pragma mark delete tags
- (void)addDeleteTagsAction:(UIAlertController *)alertController forSelectedResource:(PCResource *)selectedResource {
    __weak UIViewController *blocksafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Delete tag", nil)
                    style:UIAlertActionStyleDestructive
                andAction:^(UIAlertAction * _Nonnull action) {
                    
                    NSString *message = NSLocalizedString(@"Are you sure want to delete tag", nil);
                    
                    
                    [blocksafeSelf showOKCancelAlertWithTitle:NSLocalizedString(@"Warning", nil)
                                                   andMessage:message
                                                     okAction:^(UIAlertAction *action) {
                                                         [blocksafeSelf didBeginDoingResourceAction:deleteTagsResourceAction];
                                                         
                                                         /*
                                                          @Description -
                                                          This API is used to delete the tags for the file or folder under the specified path. Once the delete process is done/failed result will come with completion block.
                                                          
                                                          @Parameters -
                                                          PCDeleteTagsAPIParams : This class stores all the parameters required to delete the tags for the file or folder under the specified path.
                                                          * uri and accessToken are required parameter to call this API.
                                                          * For providing access token to SDK user have to logged In.
                                                          * If user will not pass required parameter, API will throw an NSInvalidArgumentException and user have to handle it.
                                                          * More information about parameters you will find in PCDeleteTagsAPIParams.h class.
                                                          
                                                          @CompletionHandler -
                                                          This completion block gets called once delete tags process is done/failed. completion block contains below parameter.
                                                          PCAPIQueryResponse : Stores all the server response and error for this API.
                                                          
                                                          @To delete the tags, you must take the following steps: -
                                                          * Include VerizonCloudAPI.framework header file in your class.
                                                          #import <VerizonCloudAPI/VerizonCloudAPI.h>
                                                          * Initalize the PCDeleteTagsAPIParams class and provide the required parameter.
                                                          * Initalize the PCAPISDK object and call the deleteTags:withCompletionHandler: API.
                                                          * Get the result(Success or failed) in completion block.
                                                          * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
                                                          */
                                                         
                                                         // Sample code for deleteTags:withCompletionHandler: API
                                                         
                                                         PCDeleteTagsAPIParams *params = [[PCDeleteTagsAPIParams alloc] init];
                                                         params.uri = selectedResource.uri;
                                                         
                                                         [[PCAPISDK sharedInstance] deleteTags:params withCompletionHandler:^(PCAPIQueryResponse *response){
                                                             
                                                             [blocksafeSelf didFinishDoingAction:deleteTagsResourceAction onResource:selectedResource result:nil response:response];
                                                         }];
                                                         
                                                         
                                                     }
                                                 cancelAction:^(UIAlertAction *action) {
                                                     
                                                 }];
                }];
}

#pragma mark -
#pragma mark Create share
- (void)addCreateShareAction:(UIAlertController *)alertController forSelectedResource:(PCResource *)selectedResource {
    __weak UIViewController *blocksafeSelf = self;
    
    __block UIAlertController *createShareAlertController = nil;
    
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Create Share", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    
                    [blocksafeSelf showCreateShareAlert:createShareAlertController forSelectedResource:selectedResource];
                }];
}

- (void)showCreateShareAlert:(UIAlertController *)alertController forSelectedResource:(PCResource *)selectedResource {
    
    __weak UIViewController *blocksafeSelf = self;
    
    alertController = [UIAlertController alertControllerWithTitle:NSLocalizedString(@"Enter a share name", nil)
                                                          message:nil
                                                   preferredStyle:UIAlertControllerStyleAlert];
    
    [self alertController:alertController showTextInputAlertWithTitle:nil andOKCompletionHandler:^(UIAlertAction *action) {
        UITextField *textField = [alertController.textFields firstObject];
        
        if (textField.text.length) {
            
            [blocksafeSelf didBeginDoingResourceAction:createShareResourceAction];
            /*
             @Description -
             This API is used to create a share. Once the create process is done/failed result will come with completion block.
             
             @Parameters -
             PCCreateShareAPIParams : This class store all the parameters required to create share.
             * resources and accessToken are required parameter to call this API. resources property holds array of PCResourceShareAPIParams object. For providing access token to SDK user have to logged In. If user will not pass required parameter, API will throw an NSInvalidArgumentException and user have to handle it.
             * name and expiresIn are  optional parameter for this API.
             * PCResourceShareAPIParams class stores all the information for resource. name and location are required parameter. Size of the resource required in case of file share only. type and subType are optional parameters.
             * More information about parameters you will find in PCCreateShareAPIParams.h and PCResourceShareAPIParams.h class.
             
             @CompletionHandler -
             This completion block gets called once create share process is done/failed. completion block contains below parameter.
             PCAPIQueryResponse :Stores all the server response and error for this API.
             PCShareItem : PCShareItem stores all the defination of a Share.
             
             @To create share, you must take the following steps: -
             * Include VerizonCloudAPI.framework header file in your class.
             #import <VerizonCloudAPI/VerizonCloudAPI.h>
             * Initalize the PCCreateShareAPIParams class and provide the required parameter.
             * resources hold array of PCResourceShareAPIParams object , Initalize the PCResourceShareAPIParams object and provide the required parameter.
             Add PCResourceShareAPIParams object in resources array.
             * Initalize the PCAPISDK object and call the createShare:withCompletionHandler: API.
             * Get the result(Success or failed) in completion block.
             * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
             */
            
            // Sample code for createShare:withCompletionHandler: API
            PCCreateShareAPIParams *params = [[PCCreateShareAPIParams alloc] init];
            params.name = textField.text;
            
            PCResourceShareAPIParams *resourceParams = [[PCResourceShareAPIParams alloc] init];
            resourceParams.name = selectedResource.name;
            resourceParams.type = ([selectedResource.file boolValue]) ? @"file" : @"folder";
            if (selectedResource.fileType == imageFileType) {
               resourceParams.subType = @"image";
            } else if (selectedResource.fileType == videoFileType) {
                resourceParams.subType = @"video";
            } else if (selectedResource.fileType == musicFileType) {
                resourceParams.subType = @"audio";
            }
            resourceParams.size = [selectedResource.size intValue];
            resourceParams.location = selectedResource.uri;
            
            params.resources = @[resourceParams];
            
            [[PCAPISDK sharedInstance] createShare:params withCompletionHandler:^(PCShareItem *shareItem, PCAPIQueryResponse *response) {
                [blocksafeSelf didFinishDoingAction:createShareResourceAction onResource:selectedResource result:shareItem response:response];
            }];
        }
    }];

}

#pragma mark -
#pragma mark Add to playlist
- (void)addToPlaylistAction:(ResourceAction)resourceAction controller:(UIAlertController *)alertController forSelectedResource:(PCResource *)selectedResource {

    
    
    __weak UIViewController *blocksafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:[self stringForResourceAction:resourceAction]
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    
                    
                    UIStoryboard *storyBoard = [UIStoryboard storyboardWithName:@"Main" bundle:nil];
                    MoveToPlaylistViewController *controller = (MoveToPlaylistViewController *)[storyBoard instantiateViewControllerWithIdentifier:@"MoveToPlaylistViewController"];
                    controller.selectedResource = selectedResource;
                    [controller setCompletionHandler:^(NSArray *playlistItems, PCAPIQueryResponse *response){
                        
                        [blocksafeSelf didFinishDoingAction:resourceAction onResource:selectedResource result:playlistItems response:response];
                    }];
                    
                    UINavigationController *navController = [[UINavigationController alloc] initWithRootViewController:controller];
                    [blocksafeSelf presentViewController:navController animated:YES completion:nil];
                }];

}

- (void)addRemoveFromPlaylistAction:(ResourceAction)resourceAction controller:(UIAlertController *)alertController forSelectedResource:(PCResource *)selectedResource {
    __weak UIViewController *blocksafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:[self stringForResourceAction:resourceAction]
                    style:UIAlertActionStyleDestructive
                andAction:^(UIAlertAction * _Nonnull action) {
                    
                    // Intialize the PCDeletePlaylistItemAPIParams object.
                    PCDeletePlaylistItemAPIParams *params = [[PCDeletePlaylistItemAPIParams alloc] init];
                    params.playlistUID = [self dataSourceForResourceAction:resourceAction];
                    params.itemUID = selectedResource.itemUID;
                    [blocksafeSelf didBeginDoingResourceAction:resourceAction];
                    /*
                     @Description :
                     This API is used to delete an item in a given playlist. Once the delete process is succeed/failed result will come with completion block.
                     
                     @Parameters :
                     PCDeletePlaylistItemAPIParams:
                     * This class stores all the parameters required to delete a playlist. It contains attributes such as itemUID and playlistUID.
                     * For providing access token to SDK user have to logged In.
                     * playlistUID and itemUID are required parameter to call this API.
                     * If user will not provide required parameter, API will throw an NSInvalidArgumentException and user have to handle it.
                     * More information about parameters you will find in PCDeletePlaylistItemAPIParams.h class.
                     
                     @CompletionHandler :
                     This completion block gets called once the delete process is succeed/failed. completion block contains below parameter.
                     PCAPIQueryResponse : Stores all the server response and error for this API.
                     
                     @To delete an item in a given playlist, you must take the following steps:
                     * Include VerizonCloudAPI.framework header file in your class.
                     #import <VerizonCloudAPI/VerizonCloudAPI.h>
                     * Initalize the PCDeletePlaylistItemAPIParams class and provide the required parameter.
                     * Initalize the PCAPISDK object and call the  deletePlaylistItem:withCompletionHandler: API.
                     * Perform the  deletePlaylistItem:withCompletionHandler: API call using the code sample below.
                     * Get the result (succeed or failed) in completion block.
                     * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
                     */

                    // Initilize the PCAPISDK and call the API.

                    [[PCAPISDK sharedInstance] deletePlaylistItem:params withCompletionHandler:^(PCAPIQueryResponse *response) {
                        // Get the result in this block.
                        [blocksafeSelf didFinishDoingAction:resourceAction onResource:selectedResource result:nil response:response];
                    }];
                }];
}



#endif
#pragma mark -
#pragma mark Add to gallery
- (void)addToGalleryAction:(ResourceAction)resourceAction controller:(UIAlertController *)alertController forSelectedResource:(PCResource *)selectedResource {
    [self alertController:alertController
       addActionWithTitle:[self stringForResourceAction:resourceAction]
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    
                    UIImage *image = [self dataSourceForResourceAction:resourceAction];
                    UIImageWriteToSavedPhotosAlbum(image, self, @selector(image:didFinishSavingWithError:contextInfo:), (__bridge void * _Nullable)(selectedResource));
                }];
}

- (void)image:(UIImage *)image didFinishSavingWithError:(NSError *)error contextInfo:(PCResource *)resource; {
    [self didFinishDoingAction:addToGalleryResourceAction onResource:resource result:nil response:error];
}


#pragma mark -
#pragma mark All action

- (void)resourceSelected:(PCResource *)selectedResource showActions:(id) firstAction, ... {
    
    // first collect the actions y iterating through them
    // duplicate actions are ignored, hence chosen a NSMutableSet
    NSMutableArray *arguments = [[NSMutableArray alloc] init];
    id eachObject;
    va_list argumentList;
    if (firstAction)  {
        [arguments addObject:firstAction];
        va_start(argumentList, firstAction);
        while ((eachObject = va_arg(argumentList, id)))  {
            [arguments addObject: eachObject];
        }
        va_end(argumentList);
    }
    
    
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:selectedResource.name
                                                                             message:nil
                                                                      preferredStyle:UIAlertControllerStyleActionSheet];
    
    
    
    for (NSNumber *actionObj in arguments) {
        switch ([actionObj intValue]) {
#ifdef KEPLER_FILE_BROWSER
                
            case copyResourceAction:
                [self addCopyAction:alertController forSelectedResource:selectedResource];
                break;
            case moveResourceAction:
                [self addMoveAction:alertController forSelectedResource:selectedResource];
                break;
            case deleteResourceAction:
                [self addDeleteAction:alertController forSelectedResource:selectedResource];
                break;
            case renameResourceAction:
                [self addRenameAction:alertController forSelectedResource:selectedResource];
                break;
            case restoreResourceAction:
                [self addRestoreAction:alertController forSelectedResource:selectedResource];
                break;
            case renameVFolderResourceAction:
                [self addRenameVirtualFolderAction:alertController forSelectedResource:selectedResource];
                break;
            case deleteVFolderResourceAction:
                [self addDeleteVFolderAction:alertController forSelectedResource:selectedResource];
                break;
#endif
#ifdef HUBBLE_MEDIA_BROWSER
            case fileInfoResourceAction: {

                [self addFileInfoAction:alertController forSelectedResource:selectedResource];
            }
                break;
            case createShareResourceAction: {
                [self addCreateShareAction:alertController forSelectedResource:selectedResource];
            }
                break;
            case setTagsResourceAction: {
                [self addSetTagsAction:alertController forSelectedResource:selectedResource];
            }
                break;
            case deleteTagsResourceAction:{
                [self addDeleteTagsAction:alertController forSelectedResource:selectedResource];
            }
                break;
            case addToPlaylistResourceAction:
            case addToAlbumResourceAction:
                [self addToPlaylistAction:[actionObj intValue] controller:alertController forSelectedResource:selectedResource];
                break;
            case removeFromPlaylistResourceAction:
            case removeFromAlbumResourceAction:
                [self addRemoveFromPlaylistAction:[actionObj intValue] controller:alertController forSelectedResource:selectedResource];
                break;
#endif
            case addToGalleryResourceAction:
                [self addToGalleryAction:[actionObj intValue] controller:alertController forSelectedResource:selectedResource];
                break;
            default:
                break;
        }
    }
    
    [self addCancelAction:alertController];
    [self presentViewController:alertController animated:YES completion:nil];
}

@end
