//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import "MoveViewController.h"
#import "UIViewController+Spinner.h"
#import "UIViewController+Alert.h"


#ifdef KEPLER_FILE_BROWSER
typedef enum {
    
    metadataGridViewType,
    trashGridViewType,
    copyGridViewType,
    moveGridViewType,
    mediaGridViewType,
} GridViewType;
#endif

static NSMutableDictionary *thumbnailImageCache;

@interface FileGridViewController ()

#ifdef KEPLER_FILE_BROWSER
@property (nonatomic, assign) GridViewType gridViewType ;
#endif

@end



@interface MoveViewController ()

- (IBAction)moveHere:(id)sender;

@end

@implementation MoveViewController

- (void)viewDidLoad {
    [super viewDidLoad];

}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

- (IBAction)moveHere:(id)sender {
    
    if (self.rootResource) {
        // Intialize the PCFOPSAPIParams object.

        PCFOPSAPIParams *params = [[PCFOPSAPIParams alloc] init];
        params.source = self.source;
        params.target = self.rootResource.fullPath;

        [self showSpinner];
        
        if (self.tabType == mediaTabType) {
            
            __weak MoveViewController *blocksafeSelf = self;
            PCResourceItem *SelectedResourceItem = [[PCResourceItem alloc] init] ;
            /*!
             @Description-This method name is moveItem,Call this method to move a file/folder from one location to a different location.
             @ Parameters - (PCFOPSAPIParams object) attributes such as source and target of the resource.
             @ReturnType - PCAPI object This return type we can cancel the ongoing request .
             @CompletionHandler-This completion block gets called on the completion of a move operation. meta data of 'target folder' is returned in completion block, Return type are (PCAPIQueryResponse object)and any type of resourceItem (PCPhoto,PCVideo,PCAudio,PCFile).
             */

            [SelectedResourceItem moveItem:params withCompletionHandler:^(id resourceItem, PCAPIQueryResponse *response) {
                [blocksafeSelf hideSpinner];
                [blocksafeSelf dismissViewControllerAnimated:YES completion:^{
                    blocksafeSelf.completionHandler(resourceItem, response);
                }];
            }];
        }
        else{
            __weak MoveViewController *blocksafeSelf = self;
            /*
             @Description :
             This API is used to move a file/folder from one location to a different location. Once the move process is succeed/failed result will come with completion block.
             
             @Parameters :
             PCFOPSAPIParams:
             * This class stores all the parameters required to move a file/folder. It contains attributes such as source, target, safe, conflictsolve and overrideRule.
             * For providing access token to SDK user have to logged In.
             * source and target are required parameter to call this API.
             * conflictsolve, safe and overrideRule are optional parameter to call this API.
             * If user will not provide required parameter, API will throw an NSInvalidArgumentException and user have to handle it.
             * More information about parameters you will find in PCFOPSAPIParams.h class.
             
             @CompletionHandler :
             This completion block gets called once the move process is succeed/failed. completion block contains below parameter.
             PCResource : This object stores metadata information of a file or a folder.
             PCAPIQueryResponse : Stores all the server response and error for this API.
             
             @To create a folder, you must take the following steps:
             * Include VerizonCloudAPI.framework header file in your class.
             #import <VerizonCloudAPI/VerizonCloudAPI.h>
             * Initalize the PCFOPSAPIParams class and provide the required parameter.
             * Initalize the PCAPISDK object and call the  move:withCompletionHandler: API.
             * Perform the  move:withCompletionHandler: API call using the code sample below.
             * Get the result (succeed or failed) in completion block.
             * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
             */
            // Initilize the PCAPISDK and call the API.
            
            [[PCAPISDK sharedInstance] move:params withCompletionHandler:^(PCResource *metadata, PCAPIQueryResponse *response) {
                // Get the result in this block.

                [blocksafeSelf hideSpinner];
                [blocksafeSelf dismissViewControllerAnimated:YES completion:^{
                    blocksafeSelf.completionHandler(metadata, response);
                }];        
            }];
            
        }
    }else {
        [self showOKAlertWithTitle:NSLocalizedString(@"Move Error", nil)
                        andMessage:[NSString stringWithFormat:@"%@", NSLocalizedString(@"You can't able to move in root folder", nil)]
                  attributedString:nil
                          okAction:nil];
    }
    
}

@end
