//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import "TrashViewController.h"
#import "UIViewController+Spinner.h"
#import "UIViewController+Alert.h"
#import "UIViewController+RightBarButtonAction.h"


@interface FileGridViewController (Protected)

@property (nonatomic, strong) NSString *emptyViewMessage;
@property (nonatomic, retain) UIRefreshControl *refreshControl;

- (void)refreshView;
- (void)alertController:(UIAlertController *)alertController
     addActionWithTitle:(NSString *)title
                  style:(UIAlertActionStyle)style
              andAction:(void (^)(UIAlertAction * _Nonnull action))handler;

@end
@interface TrashViewController ()

@end

@implementation TrashViewController

- (void)reloadData {
    
    /*
     @Description -
     This API is used to fetch a list of files and folders that have been deleted from a user repository. Once the fetch process is succeed/failed result will come with completion block.
     
     @Parameters -
     PCGetTrashAPIParams :
     * This class stores all the parameters required to retrieves a list of files and folders that have been deleted from a user repository. It contains attributes such as sort, start, count, filter and deep.
     *  For providing access token to SDK user have to logged In.
     * If user will not provide required parameter, API will throw an NSInvalidArgumentException and user have to handle it.
     * sort, start, count, filter and deep are optional parameter to call this API.
     * More information about parameters you will find in PCGetTrashAPIParams.h class.
     
     @CompletionHandler :
     This completion block gets called once the fetch process is succeed/failed. completion block contains below parameter.
     PCAPIQueryResponse :
     * Stores all the server response and error for this API.
     NSArray :
     * An array of PCMetadata object.
     
     @To fetch a list of files and folders that have been deleted from a user repository, you must take the following steps:
     * Include VerizonCloudAPI.framework header file in your class.
        #import <VerizonCloudAPI/VerizonCloudAPI.h>
     * Initalize the PCGetTrashAPIParams class and provide the required parameter.
     * Initalize the PCAPISDK object and call the fetchTrash:withCompletionHandler: API.
     * Perform the fetchTrash:withCompletionHandler: API call using the code sample below.
     * Get the result (succeed or failed) in completion block.
     * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
     */
    
    PCGetTrashAPIParams *params = [[PCGetTrashAPIParams alloc] init];
    
    __weak TrashViewController *blocksafeSelf = self;

    // Sample code for fetchTrash:withCompletionHandler: API

    
    [[PCAPISDK sharedInstance] fetchTrash:params withCompletionHandler:^(NSArray *trashItems, PCAPIQueryResponse *response) {
                
        [blocksafeSelf hideSpinner];
        [blocksafeSelf.refreshControl endRefreshing];
        
        switch (response.urlResponse.statusCode) {
            case PCAPIResponseCodeOK: {
                blocksafeSelf.dataSource = [NSArray arrayWithArray:trashItems];
                [blocksafeSelf refreshView];
            }
                break;
            case PCAPIResponseCodeUnauthorized:
                [self showUnAuthorizedAlert];
                break;
            case PCAPIResponseCodeServiceUnavailable:
                [self showServerUnavailableAlert];
                break;
            default:
                [self showOperationFailedAlert:NSLocalizedString(@"Trash fetch", nil)];
                break;
        }
    }];
}

- (void)viewDidLoad {

    [super viewDidLoad];

    if (self.rootResource) {
        self.title = self.rootResource.name;
    } else {
        self.title = NSLocalizedString(@"Trash", nil);
    }
    
    self.emptyViewMessage = NSLocalizedString(@"No items found in trash", nil);
    
    [self showSpinner];
    [self.refreshControl beginRefreshing];
    
    [self reloadData];
    // Do any additional setup after loading the view.
}

- (id)dataSourceForRightBarButtonAction:(rightBarAction)action {
    id dataSource = [super dataSourceForRightBarButtonAction:action];
    switch (action) {
        case emptyTrashRightBarAction: {
            return @"VZMOBILE";
        }
            break;
        default:
            return dataSource;
            break;
    }
}

#pragma mark -
#pragma mark RightBarButtonAction callback

- (void)didBeginDoingRightBarButtonAction:(rightBarAction)action {
    
    [self showSpinner];
}

- (void)didFinishDoingRightBarButtonAction:(rightBarAction)action :(id)result :(id)response {
    
    [self hideSpinner];
    
    PCAPIQueryResponse *APIResponse = (PCAPIQueryResponse *)response;
    
    switch (APIResponse.urlResponse.statusCode) {
        case PCAPIResponseCodeOK: {
            
            switch (action) {
                case emptyTrashRightBarAction: {
                    [self showOKAlertWithTitle:NSLocalizedString(@"Success", nil) andMessage:NSLocalizedString(@"Trash empty successfully", nil) attributedString:nil okAction:nil];
                    
                    NSMutableArray *mutableDataSource = [NSMutableArray arrayWithArray:self.dataSource];
                    [mutableDataSource removeAllObjects];
                    self.dataSource = [NSArray arrayWithArray:mutableDataSource];
                    [self refreshView];
                }
                    break;
                default:
                    break;
            }
            
        }
            break;
        case PCAPIResponseCodeUnauthorized:
            [self showUnAuthorizedAlert];
            break;
        case PCAPIResponseCodeServiceUnavailable:
            [self showServerUnavailableAlert];
            break;
        default:
            [self showOperationFailedAlert:[self stringForRightBarAction:action]];
            break;
    }
}


- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
