//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import <XCTest/XCTest.h>
#import "PCTestConstants.h"

@interface PCCreateFolderAPITest : XCTestCase

@end

@implementation PCCreateFolderAPITest

- (void)setUp {
    [super setUp];
    // Put setup code here. This method is called before the invocation of each test method in the class.
}

- (void)tearDown {
    // Put teardown code here. This method is called after the invocation of each test method in the class.
    [super tearDown];
}


/*!
 * Test Case: auth token is a required parameter. SDK should throw an exception in case auth token is nil.
 */
- (void)testNilAuthToken {
    // If User is not logged in, fetch meta data should throw an exception
    if (![[PCAPISDK sharedInstance] isUserLoggedIn]) {
        PCCreateFolderAPIParams *params = [[PCCreateFolderAPIParams alloc] init];
        XCTAssertThrows([[PCAPISDK sharedInstance] createFolder:params withCompletionHandler:^(PCResource *metadata, PCAPIQueryResponse *response) {}],@"Create folder with nil access token");
    }
}

/*!
 * Test Case: path is a required parameter. SDK should throw an exception in case path is nil.
 */
- (void) testRequiredPathForCreateFolder {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        PCCreateFolderAPIParams *params = [[PCCreateFolderAPIParams alloc] init];
        params.name = @"Test Folder";
        XCTAssertThrows([[PCAPISDK sharedInstance] createFolder:params withCompletionHandler:^(PCResource *metadata, PCAPIQueryResponse *response) {}],@"Create folder without required path");
        
    } else {
        XCTFail(@"Test Failed due to nil access token");
    }
}

/*!
 * Test Case: name is a required parameter. SDK should throw an exception in case name is nil.
 */
- (void) testRequiredNameForCreateFolder {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        PCCreateFolderAPIParams *params = [[PCCreateFolderAPIParams alloc] init];
        params.name = nil;
        params.path = @"/VZMOBILE";
        XCTAssertThrows([[PCAPISDK sharedInstance] createFolder:params withCompletionHandler:^(PCResource *metadata, PCAPIQueryResponse *response) {}],@"Create folder without required name");
        
    } else {
        XCTFail(@"Test Failed due to nil access token");
    }
}

/*!
 * Test Case: If response code is 201, Create object has to be of class PCResource type. In all other cases (error).
 */

- (void)testCreateFolder {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        PCCreateFolderAPIParams *params = [[PCCreateFolderAPIParams alloc] init];
        params.name = @"Test folder";
        params.path = @"/VZMOBILE";
        
        XCTestExpectation *expectation = [self expectationWithDescription:@"Create folder API Result"];
        
        [[PCAPISDK sharedInstance]  createFolder:params withCompletionHandler:^(PCResource *metadata, PCAPIQueryResponse *response) {
            
            switch (response.urlResponse.statusCode) {
                case PCAPIResponseCodeResourceCreated: {
                    
                    XCTAssertEqualObjects([metadata class], [PCResource class], @"Created object is not PCResource type");
                    
                }
                    break;
                    
                default:
                     XCTFail(@"Folder is not created");
                    break;
            }
             [expectation fulfill];
        }];
        [self waitForExpectationsWithTimeout:ExpectationTimeOut handler:^(NSError *error) {
            
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
        }];
        
    } else {
        XCTFail(@"Test Failed due to nil access token");
    }
    
}



@end
